/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : configuration                Source: configuration.c    */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains all functions and definitions that are      */
/*    part of the "configuration mode".                                */
/*    Whenever the user jumps into the "configuration mode" by holding */
/*    the interrupt line to a high level and send some configuration   */
/*    instructions, the functions in this module are called.           */
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include <string.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == GLOBAL DATA ==================================================== */
/*                                                                     */
unsigned char gbl_config_channel    = 5;	// Channel [Nr]: 0...38 (default 5)
unsigned char gbl_config_baud       = 5;	// Baudrate [Baud]: 0=1200, 1=2400, 2=4800, 3=9600, 4=19200, 5=38400 (default 5=38400)
unsigned char gbl_config_lnagain    = 0;	// LNA Gain [dBm]: 0=0dBm, 1=-6dBm, 2=-14dBm, 3=-20dBm (default 0=0dBm)
unsigned char gbl_config_rssi       = 1;	// RSSI Thres [dBm]: 0=-103dBm, 1=-97dBm, 2=-91dBm, 3=-85dBm, 4=-79dBm, 5=-73dBm, 6=-67dBm, 7=-61dBm (default 0=-103dBm)
unsigned char gbl_config_bandwidth  = 4;	// Bandwidth [kHz]: 0=400kHz, 1=340kHz, 2=270kHz, 3=200kHz, 4=134kHz, 5=67kHz (default 4=134kHz)
unsigned char gbl_config_dqd        = 3;	// DQD [Value]: 0...7 (default 3)
unsigned char gbl_config_power      = 7;	// Power [Value]: 0...7 (default 7)
unsigned char gbl_config_startchar  = '#';	// Start Character [Value]: 0...0xFF (default '#' or 0x23)
unsigned char gbl_config_uartbaud   = 3;	// External UART-Baudrate [Baud]: 0=1200, 1=2400, 2=4800, 3=9600, 4=14400, 5=19200, 6=28800, 7=38400, 8=57600, 9=76800, 10=115200, 11=230400, 12=250000 (default 3=9600)
unsigned char gbl_config_pollmode   = 0;	// Poll mode [Value]: 0=Poll mode disabled, 1=Poll mode active (default: 0=Poll mode disabled)
unsigned char gbl_config_fifoinput  = 8;	// Size of the FIFO buffer for received chars [Byte]: 0...10 (default: 8)
unsigned char gbl_config_fifooutput = 8;	// Size of the FIFO buffer for transfered chars [Byte]: 0...10 (default: 8)
unsigned char gbl_config_clkfrequ   = 0;    // Frequency of the external clock ouput pin: 0=10MHz, 1=5MHz, 2=3,33MHz, 3=2,5MHz, 4=2MHz, 5=1,66MHz, 6=1,25MHz, 7=1MHz (default 0=10MHz)
unsigned char gbl_config_ledsactive = 0;	// State of the LEDs: 0=Normal Operation, 1=All LEDs are off
unsigned char gbl_config_id_own     = 0;	// Identification number of the board: Standard = 0, do not use 0xAA!!
unsigned char gbl_config_id_target  = 0;    // Identification number of the target or receiver board
unsigned char gbl_config_max        = 0;	// Settings MAX3221: 0=On (normal operation), 1=On (and Autoshutdown),2=Off (default 0=On)
U_INT16       gbl_config_status     = 0;   	// The status register is initialized in the initialize.c

/* == EEPROM MEMORY ADDRESSES ======================================== */
/*                                                                     */
unsigned char ee_config_channel        EEMEM = 5;
unsigned char ee_config_baud           EEMEM = 5;
unsigned char ee_config_lnagain        EEMEM = 0;
unsigned char ee_config_rssi           EEMEM = 1;
unsigned char ee_config_bandwidth      EEMEM = 4;
unsigned char ee_config_dqd            EEMEM = 3;
unsigned char ee_config_power          EEMEM = 7;
unsigned char ee_config_startchar      EEMEM = '#';
unsigned char ee_config_uartbaud       EEMEM = 3;
unsigned char ee_config_pollmode       EEMEM = 0;
unsigned char ee_config_fifoinput      EEMEM = 8;	
unsigned char ee_config_fifooutput     EEMEM = 8;
unsigned char ee_config_ledsactive     EEMEM = 0;
unsigned char ee_config_clkfrequ       EEMEM = 0;
unsigned char ee_config_max            EEMEM = 0;
unsigned char ee_config_id_own  	   EEMEM = 0;

/* == FUNCTION configuration_main ==================================== */
/*                                                                     */
/*    This is the most important configuration function. This function */
/*    is called when the MODESELECT-Pin is high and bytes are received */
/*    via UART, SPI oder I2C.                                          */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_main(U_INT16 received_dword)
{
	unsigned char upper_byte, lower_byte;
	unsigned char dqd, rssi, bandwidth, internal_baud;
	unsigned char lna, power, external_baud, clk_frequency;
	unsigned char max;
	U_INT16 exconfigreg1 = 0x0000;
	U_INT16 exconfigreg2 = 0x0000;
	U_INT16 exconfigreg3 = 0x0000;
	unsigned char exconfigreg1_up, exconfigreg1_down, exconfigreg2_up, exconfigreg2_down;
	unsigned char exconfigreg3_down;

	upper_byte  = (received_dword) >> 8;
	lower_byte  = (unsigned char)((received_dword) & 0x00FF);

	// Instructions with no variable parameter
#ifndef DEBUG
	_delay_ms(500);
#endif
	switch(received_dword)
	{
		case 0x0000:	// Read Status Register
				configuration_status(1);
				break;
		case 0xAA00:	// Go into Sleepmode
			 	configuration_sleep();
				break;
		case 0xB500:	// External Controller needs data
				configuration_sendreceiveddata();
				break;
		case 0xBA01:	// Initialize RFM12 module
				configuration_initialize();
				break;
		case 0xBA02:	// Initialize RFM12 module with standard parameters
				configuration_fastinitialize();
				break;
		case 0xCE00:	// Send version number and compilation data
				configuration_version();
				break;
		case 0xCF01:	// Reset RFM12 and the ATmega8 to its standard settings
				configuration_resetsettings();
				configuration_fifoinputclear();	
				configuration_fifooutputclear();
				break;
		case 0xDF01:	// Save the parameters in the EEPROM 
				configuration_saveineeprom();
				break;
		case 0xEE00:	// Reset the ATmega8
				configuration_resetboard();
				break;
		case 0xF001:	// Clear FIFO input buffer
				configuration_fifoinputclear();
				break;
		case 0xF002:	// Clear FIFO output buffer
				configuration_fifooutputclear();
				break;
		default:		// Do nothing
				break;
	}

	// Instructions with one variable parameter
	switch(upper_byte)
	{
		case 0xE0:		// ID Number
				gbl_config_id_own = lower_byte;
				eeprom_write_byte(&ee_config_id_own, gbl_config_id_own);
				break;
		case 0xE1: 		// Channel Number
				gbl_config_channel = lower_byte;		
				configuration_channel();
				break;
		case 0xE2: 		// Start Character
				gbl_config_startchar = lower_byte;		
				break;	
		case 0xE3: 		// Reserved	
				break;	
		case 0xE4: 		// Size of the input FIFO buffer
				gbl_config_fifoinput = lower_byte;	
				configuration_fifoinputclear();	
				break;	
		case 0xE5: 		// Size of the output FIFO buffer
				gbl_config_fifoinput = lower_byte;		
				configuration_fifooutputclear();	
				break;	
		default:		// Do nothing
				break;
	}

	// Extended configuration register I
	if((received_dword>>13) == 0x01)
	{
		// DQD Settings
		dqd = (unsigned char) (received_dword & 0x0007);
		gbl_config_dqd = dqd;

		// RSSI Settings
		rssi = (unsigned char) ((received_dword & 0x0038) >> 3);
		gbl_config_rssi = rssi;

		// Bandwidth Settings
		bandwidth = (unsigned char) ((received_dword & 0x01C0) >> 6);
		gbl_config_bandwidth = bandwidth;

		// Internal Baudraute Settings
		internal_baud = (unsigned char) ((received_dword & 0x0E00) >> 9);
		gbl_config_baud = internal_baud;

		// Enable / disable LEDs
		if((received_dword & 0x1000))
		{
			gbl_config_ledsactive = 1;	// Turn off all LEDs
		    LED_PORT_POWERGOOD &= ~(1<<LED_PNR_POWERGOOD); // Turn off Power-Good-LED
			
		}
		else
		{
			gbl_config_ledsactive = 0;	// Normal Operation
			LED_PORT_POWERGOOD |= (1<<LED_PNR_POWERGOOD);	// Turn on Power Good LED
		}

		// Set all new configurations
		configuration_dqd();						// Set new DQD value
		configuration_rssi_bandwidth_lnagain();		// Set new RSSI and Bandwidth value
		configuration_baud();						// Set new internal baudrate
	}

	// Extended configuration register II
	if((received_dword>>13) == 0x02)
	{
		// LNA Settings
		lna = (unsigned char) (received_dword & 0x0003);
		gbl_config_lnagain = lna;

		// Power Settings
		power = (unsigned char) ((received_dword & 0x001C) >> 2);
		gbl_config_power = power;

		// External Baudrate Settings
		external_baud = (unsigned char) ((received_dword & 0x00E0) >> 5);
		gbl_config_uartbaud = external_baud;

		// Clock Frequency Settings
		clk_frequency = (unsigned char) ((received_dword & 0x0E00) >> 9);
		//clk_frequency = 5;
		gbl_config_clkfrequ = clk_frequency;
		
		// Enable / disable Pollmode for UART
		if((received_dword & 0x1000))
		{
			gbl_config_pollmode = 1;
			configuration_fifooutputclear();	// Clear FIFO input buffer
		}
		else 
			gbl_config_pollmode = 0;

		// Set all new configurations
		configuration_clkfrequency();				// Set new clock frequency		
		configuration_rssi_bandwidth_lnagain();		// Set new LNA Gain value
		configuration_power();						// Set new power value
		configuration_atmegabaudrate();				// Set new external UART baudrate
	}

	// Extended configuration register III
	if((received_dword>>13) == 0x03)
	{
		// MAX3221 Settings
		max = (unsigned char) (received_dword & 0x0007);
		gbl_config_max = max;

		// This register is not fully used
		// You can add own configuration elements

		// Set all new configurations				// Set new MAX3221 configuration
		configuration_maxrs232();
	}

	// Read configuration data and send it to the user
	switch(received_dword)
	{
		case 0xD000:	// Send ID
				if(gbl_select_interface==1)
				{
					uart_putc(0xD0);
					uart_putc(gbl_config_id_own);
				}
				// Other interfaces not implemented at this time
				break;
		case 0xD100:	// Send Channel number
				if(gbl_select_interface==1)
				{
					uart_putc(0xD1);
					uart_putc(gbl_config_channel);
				}
				// Other interfaces not implemented at this time
				break;
		case 0xD200:	// Send Startbyte
				if(gbl_select_interface==1)
				{
					uart_putc(0xD2);
					uart_putc(gbl_config_startchar);
				}
				// Other interfaces not implemented at this time
				break;			
		case 0xD400:	// Send size of the FIFO input buffer
				if(gbl_select_interface==1)
				{
					uart_putc(0xD4);
					uart_putc(gbl_config_fifoinput);
				}
				// Other interfaces not implemented at this time
				break;	
		case 0xD500:	// Send size of the FIFO output buffer
				if(gbl_select_interface==1)
				{
					uart_putc(0xD5);
					uart_putc(gbl_config_fifooutput);
				}
				// Other interfaces not implemented at this time
				break;		
		case 0xD600:	// Send Extended configuration register I
						// The content of the following 16 Bit extended config. register 
						// is absolutely equivalent to the content of the extended config.
						// register I
				if(gbl_select_interface==1)
				{
					exconfigreg1 |= gbl_config_dqd;							// Set DQD
					exconfigreg1 |= (gbl_config_rssi << 3);					// Set RSSI
					exconfigreg1 |= (gbl_config_bandwidth << 6);			// Set Bandwidth
					exconfigreg1 |= (gbl_config_baud << 9);					// Set RF12 internal baudrate
					exconfigreg1 |= (gbl_config_ledsactive << 12);			// Set LED State
					exconfigreg1 |= (1 << 13);								// Set fix log. 1

					exconfigreg1_down = (unsigned char) exconfigreg1;
					exconfigreg1_up   = (unsigned char) (exconfigreg1 >> 8);
					uart_putc(exconfigreg1_up);
					uart_putc(exconfigreg1_down);
					
				}
				// Other interfaces not implemented at this time
				break;	
		case 0xD700:	// Send Extended configuration register II
						// The content of the following 16 Bit extended config. register 
						// is absolutely equivalent to the content of the extended config.
						// register II
				if(gbl_select_interface==1)
				{
					exconfigreg2 |= gbl_config_lnagain;						// Set LNA Gain
					exconfigreg2 |= (gbl_config_power << 2);				// Set Power
					exconfigreg2 |= (gbl_config_uartbaud << 5);				// Set UART Baud
					exconfigreg2 |= (gbl_config_clkfrequ << 9);				// Set ext. clock frequency
					exconfigreg2 |= (gbl_config_pollmode << 12);			// Set Pollmode
					exconfigreg2 |= (1 << 14);								// Set fix log. 1

					exconfigreg2_down = (unsigned char) exconfigreg2;
					exconfigreg2_up   = (unsigned char) (exconfigreg2 >> 8);
					uart_putc(exconfigreg2_up);
					uart_putc(exconfigreg2_down);
				}
				// Other interfaces not implemented at this time
				break;	
		case 0xD800:	// Send Extended configuration register III
						// The content of the following 16 Bit extended config. register 
						// is absolutely equivalent to the content of the extended config.
						// register III
				if(gbl_select_interface==1)
				{
					exconfigreg3 |= gbl_config_max;							// Set MAX3221 state
					exconfigreg3 |= (1 << 13);								// Set fix log. 1
					exconfigreg3 |= (1 << 14);								// Set fix log. 1

					exconfigreg3_down = (unsigned char) exconfigreg3;
					uart_putc(exconfigreg3_down);
				}
				// Other interfaces not implemented at this time
				break;
		}
}



/* == FUNCTION configuration_sleep =================================== */
/*                                                                     */
/*    This function sends the AVR and the RFM12 module into a deep     */
/*    sleep mode. All operations stop. You can only reactivate the     */
/*    AVR and the RFM12 by pulling the Interrupt Pin to a low level.   */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_sleep(void)
{
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);	// Select Power Down for sleepmode
  	MCUCR &= ~(1<<ISC00);   				// Interrupt is generated on low 
	GICR |= (1<<INT0); 						// Enable the external interrupt
  	_delay_ms(500);							// Wait some milliseconds
	set_leds(LED_POWERGOOD,LED_OFF);		// Turn of the power good LED	
	rf12_sleep();							// RFM12 module sleeps
  	sleep_mode();							// AVR sleeps
	_delay_ms(10);							// Wait some ms after wakeup
	rf12_wakeup();							// Wakeup RFM12
	set_leds(LED_POWERGOOD,LED_ON);			// Turn the power good LED  	
	GICR &= ~(1<<INT0); 					// Disable the external interrupt	
}

/* == FUNCTION configuration_fastinitialize ========================== */
/*                                                                     */
/*    This function provides a "plug and play" configuration for the   */
/*    board. It initializes the module with standard parameters.       */
/*    For most purposes this standard parameters are very good         */
/*    and you don't need to reconfigurate this parameters. You can     */
/*    initialize your board with this function and test the wireless   */
/*    communication for the first time.                                */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_fastinitialize(void)
{
	rf12_init();						// Initialize the module
	rf12_setchannel(5);					// Set channel 5
    rf12_setreccontrol(134,0,-103);		// Some other settings, look at rf12.c
	rf12_setbaudrate(BAUD38400); 		// We use 38400 Baud
	rf12_setdqdreport(3);				// Set DQD report to 3
	rf12_setpower(7);					// Set maximum power
	_delay_ms(2);						// Wait 2ms
}


/* == FUNCTION configuration_initialize ============================== */
/*                                                                     */
/*    This function only initialize the RFM12 module.                  */
/*    The module need some other settings to work good.                */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_initialize(void)
{
	rf12_init();	// (Re-)initialize the RFM12 module
}


/* == FUNCTION configuration_channel ================================= */
/*                                                                     */
/*    This function sets the channel number                            */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_channel(void)
{
	rf12_setchannel(gbl_config_channel);	// Transfer the value to the rf12-function
}


/* == FUNCTION configuration_baud ==================================== */
/*                                                                     */
/*    This function configures the baudrate of the RFM12 module.       */
/*    It is NOT the boardrate of the hardware UART of the AVR!!        */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_baud(void)
{
	BAUDRATE baud;

	switch(gbl_config_baud)			// Convert the baudrate value
	{	
		case 0: baud = BAUD1200;
				break;
		case 1: baud = BAUD2400;
				break;
		case 2: baud = BAUD9600;
				break;
		case 3: baud = BAUD19200;
				break;
		case 4: baud = BAUD38400;
				break;
		case 5: baud = BAUD57600;
				break;
		case 6: baud = BAUD69000;
				break;
		case 7: baud = BAUD115200;
				break;
		default:baud = BAUD9600;
				break;
	}
	rf12_setbaudrate(baud); // Transfer the value to the rf12-function
}


/* == FUNCTION configuration_dqd ===================================== */
/*                                                                     */
/*    This function sets the DQD "Good Signal Quality" identifier.     */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_dqd(void)
{
	rf12_setdqdreport(gbl_config_dqd);	// Transfer the value to the rf12-function
}


/* == FUNCTION configuration_dqd ===================================== */
/*                                                                     */
/*    This function sets the power for the antenna output on the RFM12 */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_power(void)
{
	rf12_setpower(gbl_config_power);	// Transfer the value to the rf12-function
}

/* == FUNCTION configuration_clkfrequency ============================ */
/*                                                                     */
/*    This function sets the frequency for the clock output pin CLK    */
/*    of the RFM12 module.                                             */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_clkfrequency(void)
{
	rf12_setclk(gbl_config_clkfrequ);	// Transfer the value to the rf12-function
}


/* == FUNCTION configuration_rssi_bandwidth_lnagain ================== */
/*                                                                     */
/*    This function sets the parameter for bandwidth, rssi and LNA     */
/*    gain. The function is called when one of the 3 parameters is     */
/*    changed by the user.                                             */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_rssi_bandwidth_lnagain(void)
{
	signed char rssi,lnagain;
	S_INT16	bandwidth;

	switch(gbl_config_rssi)		// Convert the RSSI value
	{
		case 0: rssi=-103;
				break;
		case 1: rssi=-97;
				break;
		case 2: rssi=-91;
				break;
		case 3: rssi=-85;
				break;
		case 4: rssi=-79;
				break;
		case 5: rssi=-73;
				break;
		case 6: rssi=-67;
				break;
		case 7: rssi=-61;
				break;
		default:rssi=-103;
				break;
	}
	
	switch(gbl_config_bandwidth)	// Convert the Bandwidth value
	{
		case 0:	bandwidth=400;
				break;
		case 1: bandwidth=340;
				break;
		case 2: bandwidth=270;
				break;
		case 3: bandwidth=200;
				break;
		case 4:	bandwidth=134;
				break;
		case 5: bandwidth=67;
				break;
		default:bandwidth=134;
				break;
	}

	switch(gbl_config_lnagain)	// Convert the LNA Gain value
	{
		case 0: lnagain=0;
				break;
		case 1: lnagain=-6;
				break;
		case 2: lnagain=-14;
				break;
		case 3: lnagain=-20;
				break;
		default:lnagain=0;
				break;
	}

	rf12_setreccontrol(bandwidth,lnagain,gbl_config_rssi);	// Transfer the converted values to the rf12-functions
}


/* == FUNCTION configuration_resetsettings =========================== */
/*                                                                     */
/*    This function reset all settings to its standard values.         */
/*    All previous settings will be overwritten with the standard      */
/*    values. In the following step all neccessary configuration       */
/*    functions are called to set the standard values immediately.     */
/*    After about 4ms all things are done.                             */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_resetsettings(void)
{
	//First set all global parameters to its standard value
	gbl_config_channel    = 5;
	gbl_config_baud       = 5;	
	gbl_config_lnagain    = 0;
	gbl_config_rssi       = 0;
	gbl_config_bandwidth  = 4;	
	gbl_config_dqd        = 3;
	gbl_config_power      = 7;	
	gbl_config_startchar  = '#';
	gbl_config_uartbaud   = 3;	
	gbl_config_pollmode   = 0;	
	gbl_config_fifoinput  = 8;	
	gbl_config_fifooutput = 8;	
	gbl_config_clkfrequ   = 0;  
	gbl_config_ledsactive = 0;	

	
	//Then write the parameters into the RFM12 module by calling every single function
	rf12_init();								// Initialize the module a second time
	rf12_setchannel(5);							// Set channel to 5
	rf12_setreccontrol(134,0,-103);				// 134KHz Bandwidth, 0dBm LNA Gain, -103dBm RSSI Threshold
	rf12_setbaudrate(BAUD38400); 				// Baudrate 38400 Baud
	rf12_setdqdreport(3);						// DQD report 3
	rf12_setpower(7);							// Maximum antenna power
#ifndef DEBUG
	_delay_ms(2);								// Short delay
#endif

	//Configure the UART baudrate to its standard
	configuration_atmegabaudrate();

	//Set Clock Frequency to 10MHz
	configuration_clkfrequency();				

	//Reset the FIFO output buffer
	gbl_radio_output_locnextfreepack = 0;		// FIFO buffer is now empty
	gbl_dataavail_type = 0;						// No data is available
	gbl_rcintcounter = 0;						// No received data via UART, SPI or I2C in process

	//Reset the FIFO input buffer
	gbl_radio_input_locnextfreepack = 0;		// FIFO buffer is now empty

	//Reset the error handler
	error_handler(NO_ERROR);					// There is no error
}

/* == FUNCTION configuration_saveineeprom ============================ */
/*                                                                     */
/*    This function stores the actual values of all configuration      */
/*    parameters in the internal EEPROM of the ATmega8.                */
/*    They can be simply loaded by sending the configuration word      */
/*    0xDF02. This works at any time.                                  */
/*                                                                     */
/*    The following parameters are stored in the EEPROM:               */
/*                                                                     */
/*       - Channel Number                  - Startchar                 */
/*       - Internal Baudrate               - Amout of char to trans.   */
/*       - LNA Gain                        - External UART baudrate    */
/*       - RSSI Threshold                  - Pollmode                  */
/*       - Bandwidth                       - State of the UART, I2C    */
/*       - DQD quality report                and SPI                   */
/*       - Power of the antenna output                                 */
/*                                                                     */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_saveineeprom(void)
{
	// Disable the global interrupts
	cli();

	// Store the values
	eeprom_write_byte(&ee_config_channel, gbl_config_channel);
	eeprom_write_byte(&ee_config_baud, gbl_config_baud);
	eeprom_write_byte(&ee_config_lnagain, gbl_config_lnagain);
	eeprom_write_byte(&ee_config_rssi, gbl_config_rssi);
	eeprom_write_byte(&ee_config_bandwidth, gbl_config_bandwidth);
	eeprom_write_byte(&ee_config_dqd, gbl_config_dqd);
	eeprom_write_byte(&ee_config_power, gbl_config_power);
	eeprom_write_byte(&ee_config_startchar, gbl_config_startchar);
	eeprom_write_byte(&ee_config_uartbaud, gbl_config_uartbaud);
	eeprom_write_byte(&ee_config_pollmode, gbl_config_pollmode);
	eeprom_write_byte(&ee_config_fifoinput, gbl_config_fifoinput);
	eeprom_write_byte(&ee_config_fifooutput, gbl_config_fifooutput);
	eeprom_write_byte(&ee_config_ledsactive, gbl_config_ledsactive);
	eeprom_write_byte(&ee_config_clkfrequ, gbl_config_clkfrequ);

}

/* == FUNCTION configuration_loadfromeeprom ========================== */
/*                                                                     */
/*    This function loads the previous saved settings in the internal  */
/*    EEPROM of the AVR and overwrite the actual values.               */
/*    With this feature you can simply reload old values.              */
/*                                                                     */
/*    The following parameters are loaded out of the EEPROM:           */
/*                                                                     */
/*       - Channel Number                  - Startchar                 */
/*       - Internal Baudrate               - Amout of char to trans.   */
/*       - LNA Gain                        - External UART baudrate    */
/*       - RSSI Threshold                  - Pollmode                  */
/*       - Bandwidth                       - State of the UART, I2C    */
/*       - DQD quality report                and SPI                   */
/*       - Power of the antenna output                                 */
/*                                                                     */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_loadfromeeprom(void)
{
	// Disable the global interrupts
	cli();


	// Load the values from the EEPROM
	gbl_config_channel    = eeprom_read_byte(&ee_config_channel); 
	gbl_config_baud       = eeprom_read_byte(&ee_config_baud); 
	gbl_config_lnagain    = eeprom_read_byte(&ee_config_lnagain); 
	gbl_config_rssi       = eeprom_read_byte(&ee_config_rssi); 
	gbl_config_bandwidth  = eeprom_read_byte(&ee_config_bandwidth); 
	gbl_config_dqd        = eeprom_read_byte(&ee_config_dqd); 
	gbl_config_power      = eeprom_read_byte(&ee_config_power); 
	gbl_config_startchar  = eeprom_read_byte(&ee_config_startchar); 
	gbl_config_uartbaud   = eeprom_read_byte(&ee_config_uartbaud); 
	gbl_config_pollmode   = eeprom_read_byte(&ee_config_pollmode); 
	gbl_config_fifoinput  = eeprom_read_byte(&ee_config_fifoinput);
	gbl_config_fifooutput = eeprom_read_byte(&ee_config_fifooutput);
	gbl_config_ledsactive = eeprom_read_byte(&ee_config_ledsactive);
	gbl_config_clkfrequ   = eeprom_read_byte(&ee_config_clkfrequ);

	//Configure the ATmega32 and RFM12 module with the EEPROM parameters
	configuration_channel();					//  1. Set Channel
	_delay_us(10);				
	configuration_rssi_bandwidth_lnagain();		//  2. Set RSSI, Bandwidth, LNA Gain
	_delay_us(10);
	configuration_baud();						//  3. Set RF12 internal baudrate
	_delay_us(10);
	configuration_dqd();						//  4. Set DQD report
	_delay_us(10);
	configuration_power();						//  5. Set Power state
	_delay_us(10);
	configuration_atmegabaudrate();				//  6. Set UART baudrate
	configuration_fifoinputclear();				//  7. Clear FIFO input buffer
	configuration_fifooutputclear();			//  8. Clear FIFO output buffer
	configuration_clkfrequency();				// 10. Set Clock output frequency


}



/* == FUNCTION configuration_resetboard ============================== */
/*                                                                     */
/*    This function uses the watchdog timer to reset the ATmega8       */
/*    just like with a "real hardware reset".                          */
/*    Notice that all parameters and the RFM12 module are set to its   */
/*    standard values.                                                 */
/*    The time needed to reset the ATmega8 is about 15ms.              */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_resetboard(void)
{
	wdt_enable(WDTO_15MS);			// Set Watchdogtimer to 15ms
	_delay_ms(300);					// Wait 300ms
}


/* == FUNCTION configuration_atmegabaudrate ========================== */
/*                                                                     */
/*    This function reinitialize the hardware-UART of the AVR with     */
/*    the selected baudrate.                                           */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_atmegabaudrate(void)
{
	switch(gbl_config_uartbaud)
	{
		case 0: uart_init( UART_BAUD_SELECT(1200,F_CPU) );
				break;
		case 1: uart_init( UART_BAUD_SELECT(2400,F_CPU) );
				break;
		case 2: uart_init( UART_BAUD_SELECT(4800,F_CPU) );
				break;
		case 3: uart_init( UART_BAUD_SELECT(9600,F_CPU) );
				break;
		case 4: uart_init( UART_BAUD_SELECT(14400,F_CPU) );
				break;
		case 5: uart_init( UART_BAUD_SELECT(19200,F_CPU) );
				break;
		case 6: uart_init( UART_BAUD_SELECT(28800,F_CPU) );
				break;
		case 7: uart_init( UART_BAUD_SELECT(38400,F_CPU) );
				break;
		case 8: uart_init( UART_BAUD_SELECT(57600,F_CPU) );
				break;
		case 9: uart_init( UART_BAUD_SELECT(76800,F_CPU) );
				break;
		case 10:uart_init( UART_BAUD_SELECT(115200,F_CPU) );
				break;
		case 11:uart_init( UART_BAUD_SELECT(230400,F_CPU) );
				break;
		case 12:uart_init( UART_BAUD_SELECT(250000,F_CPU) );
				break;
		default: // Do nothing
				break;
	}

	// Define the timer2 interrupt time
	// This determines how long the programm should wait for new incomming serial data
	// before the received characters before are manipulated. (Only in variable char length mode)
	gbl_INT_Timer2_Counter_Max = compute_bitrate(gbl_config_uartbaud); 
}

/* == FUNCTION configuration_sendreceiveddata ======================== */
/*                                                                     */
/*    This function only works for "pollmode-interfaces" like I2C and  */
/*    SPI and UART (if the pollmode-bit is set).                       */
/*    This function sends the first received character from the RFM12  */
/*    to an external microcontroller and update the FIFO register.     */
/*                                                                     */
/*    How it works:                                                    */
/*    We assume that the size of our FIFO input buffer is 4, so we     */
/*    have a 2-dimensional char array. A array with 4 elements, each   */
/*    element is a array of 50 chars.                                  */
/*                                                                     */
/*         -------------                                               */
/*    0    |0x00...0x31|                                               */
/*         -------------                                               */
/*    1    |0x00...0x31|                                               */
/*         -------------                                               */
/*   ...   |0x00...0x31|                                               */
/*         -------------                                               */
/*   10    |0x00...0x31|                                               */
/*         -------------                                               */
/*                                                                     */
/*    Every time a character is received from the RFM12 it is stored   */
/*    in the FIFO input buffer you see below. The first received       */
/*    string is stored in element 0, the second received string in     */
/*    element 1 and so on...                                           */
/*    When the external microcontroller wants to get a received byte   */
/*    the data in element 0 is transfered to the microcontroller (we   */
/*    assume that element 0 contains data). After this all other       */
/*    elements are shiftet. The data of element 1 is copied into       */
/*    element 0, the data of element 2 is copied into element 1 and so */
/*    on.                                                              */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_sendreceiveddata(void)
{
	unsigned char i,k;

	if(gbl_radio_input_locnextfreepack==0)
	{
		// No data in fifo input buffer
		if((gbl_select_interface==1) && (gbl_config_pollmode==1))
			//uart_puts("NO DATA");	
			;//Do nothing, send nothing
		if(gbl_select_interface==2)
			;//TODO
		if(gbl_select_interface==3)
			;//TODO
	}
	else if(gbl_radio_input_locnextfreepack>=1)
	{
		// Variable char length mode
		// Data in fifo input buffer
		// Read the first received string
		if((gbl_select_interface==1) && (gbl_config_pollmode==1))
		{
			uart_putc(gbl_config_id_target);
			for(i=0;i<gbl_radio_input_charnumber[0];i++)	
				uart_putc(gbl_radio_input_muxbuffer[0][i]);
		}
		if(gbl_select_interface==2)
			;//TODO
		if(gbl_select_interface==3)
			;//TODO

		// Shift FIFO buffer
   		for(i=0;i<gbl_radio_input_locnextfreepack;i++)
		{
			for(k=0;k<=gbl_radio_input_charnumber[i+1];k++)
				gbl_radio_input_muxbuffer[i][k] = gbl_radio_input_muxbuffer[i+1][k];
			
			gbl_radio_input_charnumber[i] = gbl_radio_input_charnumber[i+1];
		}
		
			// Decrease the counter
		gbl_radio_input_locnextfreepack--;
			if(gbl_radio_input_locnextfreepack==0)			//Check if the buffer is now empty
			CTRL_PORT_RECC &= ~(1<<CTRL_PNR_RECC);	//If the buffer is empty, set the receive-complete-pin to low-level
	}	
}

/* == FUNCTION configuration_fifoinputclear ========================== */
/*                                                                     */
/*    This function clears the FIFO input buffer                       */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_fifoinputclear(void)
{
	gbl_radio_input_locnextfreepack = 0; // Set the counter for the FIFO input buffer back to zero

	// The FIFO input buffer is now empty
}

/* == FUNCTION configuration_fifooutputclear ========================= */
/*                                                                     */
/*    This function clears the FIFO output buffer                      */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_fifooutputclear(void)
{
	gbl_rcintcounter = 0;				  // Set the interrupt counter to zero			
	gbl_radio_output_locnextfreepack = 0; // Set the counter for the FIFO output buffer back to zero

	// The FIFO output buffer is now empty
}


/* == FUNCTION configuration_fifooutputclear ========================= */
/*                                                                     */
/*    This function sends the version number and the compilation date  */
/*    of the programm. This function is only supported by the UART.    */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_version(void)
{	
	if(gbl_select_interface==1)
	{
		uart_puts(VERSION);		// Send the version number
		uart_puts(__DATE__);	// Send the compilation date
	}
}


/* == FUNCTION configuration_maxrs232 ================================ */
/*                                                                     */
/*    This function enables or disables the MAX3221 on the FUNKBOARB.  */
/*    There are 2 different modes with can be selected:                */
/*                                                                     */
/*    ON and NORMAL  OPERATION:                                        */
/*    The MAX3221 is in use, use RX232 and TX232 pins. The feature     */
/*    Auto-Shutdown is NOT ACTIVATED.                                  */
/*                                                                     */
/*    ON and AUTO SHUTDOWN:                                            */
/*    The MAX3221 is in use, use RX232 and TX232 pins. The feature     */
/*    Auto-Shutdown is also activated. Have a look on the datasheet    */
/*    of the MAX3221 for more information.                             */
/*                                                                     */
/*    OFF:                                                             */
/*    The MAX3221 is not enabled, use RX and TX pins.                  */
/*                                                                     */
/*    The mode of the MAX3221 is independent from the UART.            */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_maxrs232(void)
{	
	switch(gbl_config_max)
	{
		case 0:	// MAX3221 is on (normal operation)
				MAX_PORT_MAX3221 &= ~(1<<MAX_PNR_ENABLE); 		// EN=0
				MAX_PORT_MAX3221 |=  (1<<MAX_PNR_FORCEOFF);		// FORCEOFF=1
				MAX_PORT_MAX3221 |=  (1<<MAX_PNR_FORCEON);		// FORCEON=1
				break;	
		case 1:	// MAX3221 is on (autoshutdown)
				MAX_PORT_MAX3221 &= ~(1<<MAX_PNR_ENABLE); 		// EN=0
				MAX_PORT_MAX3221 |=  (1<<MAX_PNR_FORCEOFF);		// FORCEOFF=1
				MAX_PORT_MAX3221 &= ~(1<<MAX_PNR_FORCEON);		// FORCEON=0
				break;
		case 2:	// MAX3221 is off
				MAX_PORT_MAX3221 |=  (1<<MAX_PNR_ENABLE); 		// EN=1
				MAX_PORT_MAX3221 &= ~(1<<MAX_PNR_FORCEOFF);		// FORCEOFF=0
				MAX_PORT_MAX3221 &= ~(1<<MAX_PNR_FORCEON);		// FORCEON=0
				break;		
	}		
}


/** END OF MODULE ******************************************************/
