/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : error_handler                Source: error_handler.c    */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains the error handler. Every "well known" error */
/*    that can occur in the programm is noticed by the error handler   */
/*    the error handler receives the error message of the last error   */
/*    and saves the number of this message in the status register.     */
/*    So the user can have a look on the errors that occur in the      */
/*    system. If more than a specific number of errors occur in the    */
/*    running system the error handler set a LED to signalize the      */
/*    user that something goes wrong.                                  */
/*    If a fatal error occurs the error handler can save the actual    */
/*    settings and reset the whole system.                             */
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == GLOBAL DATA ==================================================== */
/*                                                                     */

/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */

/* == FUNCTION error_handler ========================================= */
/*                                                                     */
/*    The error handler function saves tha last occured error in the   */
/*    programm. Every 'well known' error has got its own binary error  */
/*    code.                                                            */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
unsigned char error_handler(ERRORCODE errormsg)
{
	static unsigned char error_number;

	switch(errormsg)
	{
		case NO_ERROR:                // NO ERROR REPORTED	 
									  error_number = 0;
					                  break;
		case FRAMEERROR:        	 // INTERFACE: FRAME ERROR
									  error_number = 1;
					                  break;
		case OVERRUNERROR:      	 // INTERFACE: UART OVERRUN ERROR
									  error_number = 2;
					                  break;		
		case BUFFEROVERFLOW:    	 // INTERFACE: UART BUFFER OVERFLOW
									  error_number = 3;
					                  break;			
		case RADIO_DATANOTVALID:      // RADIO: RECEIVED RADIO DATA NOT VALID
									  error_number = 4;
					                  break;
		case RADIO_INBUFFEROVERFLOW:  // RADIO: FIFO INPUT BUFFER OVERFLOW
									  error_number = 5;
					                  break;
		case RADIO_OUTBUFFEROVERFLOW: // RADIO: FIFO OUTPUT BUFFER OVERFLOW
									  error_number = 6;
					                  break;
		case TRANSMIT_TIMEOUT:	      // ACKNOWLEDGE RESPONSE: NO RESPONSE! RUN INTO TIMEOUT
									  error_number = 7;
									  break;
		case TRANSMIT_ERROR:		  // ACKNOWLEDGE RESPONSE: RESPONSE, BUT PACKAGE ERROR
									  error_number = 8;
									  break;
		case TRANSMIT_NACK:			  // ACKNOWLEDGE RESPONSE: NOT ACKNOWLEDGE
								      error_number = 9;
									  break;
		case TRANSMIT_SOMETRY:		  // ACKNOWLEDGE RESPONSE: MORE THAN ONE ATTEMPT TO TRANSFER PACKAGE, BUT ALL OK
									  error_number = 10;
									  break;
		case DUMMY:					  return(11);
	}

	return error_number;
}



/** END OF MODULE ******************************************************/
