
/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : initialize                   Source: initialize.c       */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains the full initializing sequence of the       */
/*    programm. The function in this module is called before all       */
/*    other functions in the whole programm (excepts main).            */
/*    In this module ALL used interrupts, timer, etc. are initialized. */
/*    But maybe they are not activated.                                */
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == GLOBAL DATA ==================================================== */
/*                                                                     */

/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */


/* == FUNCTION initialize ============================================ */
/*                                                                     */
/*    This function initializes all peripherals and components of the  */
/*    ATmega32 and RFM12.                                              */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void initialize(void)
{
	/* Wait a few secnds after power-on   */
	/* ---------------------------------- */
#ifndef DEBUG
	_delay_ms(500);							// 500ms are enough for almost all voltage regulators
#endif

	/* Disable all interrupts             */
	/* ---------------------------------- */
	cli();

	/* Initialize the UART interface      */
	/* ---------------------------------- */
	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );	// Baud 9600, No Parity, 8 Databits, 1 Stopbit	
	UCSRB |= (1<<RXEN);		        		// Activate UART receive complete interrupt
	UCSRB |= (1<<RXCIE);					// Activate UART receive complete interrupt

	/* Initialize the SPI interface       */
	/* ---------------------------------- */
	// TODO	

	/* Initialize the I2C interface       */
	/* ---------------------------------- */
    //	

	/* Initialize the Timeout Timer0      */
	/* ---------------------------------- */
	TCCR0 |= (1<<WGM01);					// Timer in CTC mode
    TCCR0 &= ~((1<<CS00)|(1<<CS01)|(1<<CS02));// Stop Timer0
	OCR0 = 0xFF;							// Define maximum value
	TIMSK &= ~(1<<OCIE0);					// Disable timer0 interrupt at the initializing

	/* Initialize the Misc Timer1         */
	/* ---------------------------------- */
	TCCR1B |= (1<<WGM12);					// Timer in CTC mode
	OCR1A = 0xFF;							// Define maximum value
    TCCR1B &= ~ ((1<<CS10)|(1<<CS11)|(1<<CS12));// Stop Counter
	TIMSK &= ~(1<<OCIE1A);					// Disable timer1 interrupt at the initializing


	/* Initialize the Timer2              */
	/* ---------------------------------- */
	TCCR2 |= (1<<WGM21);					// Timer in CTC mode
    TCCR2 &= ~((1<<CS20)|(1<<CS21)|(1<<CS22));// Stop Counter
	TIMSK &= ~(1<<OCIE2);					// Disable timer2 interrupt at the initializing
	OCR2 = 0xFF;							// Define maximum value
	gbl_INT_Timer2_Counter_Max = compute_bitrate(5); // Define the timer2 interrupt time


	/* Initialize the RFM12 module        */
	/* ---------------------------------- */
	rf12_init();							// Initializing routine
	rf12_setchannel(5);						// Channel 5
    rf12_setreccontrol(134,0,-103);			// 134KHz Bandwidth, 0dBm LNA Gain, -103dBm RSSI Threshold
	rf12_setbaudrate(BAUD57600); 			// Baudrate 57600
	rf12_setdqdreport(3);					// DQD = 3
	rf12_setpower(7);						// Maximum antenna output
#ifndef DEBUG	
	_delay_ms(2);							// Short delay 
#endif

	/* Initialize external Interrupts     */
	/* ---------------------------------- */	
	// Interrupts are still enabled	
	// Configure INT0 as external interrupt
  	CTRL_DDR_MDSEL  &= ~(1<<CTRL_PNR_MDSEL);// configure MODESELECT as input
 	MCUCR |= (1<<ISC00);    				// Interrupt is generated on low 
  	GICR |= (1<<INT0);	    				// Disable the Interrupt routine

	/* Initialize the MODESELECT Pin      */
	/* ---------------------------------- */
	CTRL_PORT_RECC &= ~(1<<CTRL_PNR_RECC);	// Receive-Complete-Pin as Output (low)		
	CTRL_DDR_RECC |= (1<<CTRL_PNR_RECC);

	/* Initialize the status register     */
	/* ---------------------------------- */
	configuration_status(0);				// Set configuration register, but do not send its content

	/* Initialize the error_hander        */
	/* ---------------------------------- */	
	error_handler(NO_ERROR);				// Init error handler with no error

	/* Load the ID of the Board           */
	/* ---------------------------------- */
	gbl_config_id_own = eeprom_read_byte(&ee_config_id_own); 	// Load the ID from the internal EEPROM of the AVR

	/* Initialize the LBF pin             */
	/* ---------------------------------- */	
	ZLDO_DDR_LBF  &= ~(1<<ZLDO_PNR_LBF);				// Define LBF as input
	ZLDO_PORT_LBF &= ~(1<<ZLDO_PNR_LBF);				// No pullup

	/* Initialize the SC pin              */
	/* ---------------------------------- */	
	ZLDO_DDR_SC  &= ~(1<<ZLDO_PNR_SC);				// Define SC as input
	ZLDO_PORT_SC &= ~(1<<ZLDO_PNR_SC);				// No pullup

	/* Initialize the MAX3221 control pins*/
	/* ---------------------------------- */
		// Deactivate MAX3221 at startup
	MAX_PORT_MAX3221 |= (1<<MAX_PNR_ENABLE); 		// EN=1
	MAX_PORT_MAX3221 &= ~ (1<<MAX_PNR_FORCEOFF);	// FORCEOFF=0
	MAX_PORT_MAX3221 |= (1<<MAX_PNR_FORCEON);		// FORCEON=1

	MAX_DDR_MAX3221 |= (1<<MAX_PNR_ENABLE);			// Define ENABLE   as output
	MAX_DDR_MAX3221 |= (1<<MAX_PNR_FORCEOFF);		// Define FORCEOFF as output
	MAX_DDR_MAX3221 |= (1<<MAX_PNR_FORCEON);		// Define FORCEON  as output
	MAX_DDR_MAX3221 &=~(1<<MAX_PNR_INVALID);		// Define INVALID  as input
 
	gbl_select_interface = 1;						// Activate UART

		// Check if a valid RS232 signal
		// is applied to the MAX3221
	if(MAX_PIN_INVALID & (1<<MAX_IPN_INVALID))
	{
		// Valid signal detected, MAX is on
		// (and autoshutdown is deactivated)
		gbl_config_max = 0;	
		configuration_maxrs232();
	}
	else
	{
		// No valid signal detected, MAX is off
		gbl_config_max = 2;
		configuration_maxrs232();
	}
	 	
	/* Load parameters from EEPROM        */
	/* ---------------------------------- */

#ifdef Hausnummer1
        gbl_config_id_own = 1;
        eeprom_write_byte(&ee_config_id_own, gbl_config_id_own);
        configuration_saveineeprom();
#endif
#ifdef Hausnummer2
        gbl_config_id_own = 2;
        eeprom_write_byte(&ee_config_id_own, gbl_config_id_own);
        configuration_saveineeprom();
#endif

	configuration_loadfromeeprom();			// Set this function all other times


	/* Turn on the power-good LED         */
	/* ---------------------------------- */
	LED_DDR_POWERGOOD |= (1<<LED_PNR_POWERGOOD);	// Configure all LEDs as output
	LED_DDR_RECEIVING |= (1<<LED_PNR_RECEIVING);
	LED_DDR_TRANSMITT |= (1<<LED_PNR_TRANSMITT);

	/* Initializing OK, turn on all LEDs  */
	/* ---------------------------------- */
	set_leds(LED_POWERGOOD,LED_ON);			
	set_leds(LED_RECEIVING,LED_ON);
	set_leds(LED_TRANSMITT,LED_ON);
#ifndef DEBUG
	_delay_ms(500);
#endif
	set_leds(LED_POWERGOOD,LED_ON);
	set_leds(LED_RECEIVING,LED_OFF);
	set_leds(LED_TRANSMITT,LED_OFF);

	/* Turn on global interrupts          */
	/* ---------------------------------- */
	sei();									// Global Interrupts enabled


	// INITIALIZING FINISHED !!
}



/** END OF MODULE ******************************************************/



