/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : interrupts                   Source: interrupts.c       */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains all interrupt functions plus some other     */
/*    functions that are used by the interrupt routines.               */
/*                                                                     */
/*    HELPFILE: THE FIFO OUTPUT BUFFER AND CONFIGURATION BUFFER        */
/*    The functions in this file mainly control the output buffers.    */
/*    there are 2 types of commands/data strings you can send to the   */
/*    ATmega. You can send configuration data, to control the behavior */
/*    of the program and you can send data that is trasmitted by the   */
/*    RFM12 module to other boards.                                    */
/*                                                                     */
/*    These different commands possess different buffers to store the  */
/*    last received values.                                            */
/*                                                                     */
/*    In configuration mode the programm works with a double-buffer.   */
/*    Every received character (from UART, I2C or SPI) is stored in a  */
/*    first buffer. When the configuration command (2 characters) is   */
/*    completely received, the whole command is copied in a second     */
/*    buffer. So the programm can work with the fully received command */
/*    (that is in the second buffer) while the interrupt routine can   */
/*    receive the next data string.                                    */
/*                                                                     */
/*    In radio transmission mode the programm works with a adjustable  */
/*    FIFO buffer. The size of the FIFO buffer can be determined by    */
/*    the user. When characters are received the interrupt routine     */
/*    stores this characters in a first buffer. When a complete data   */
/*    string is received this data string is copied in the FIFO        */
/*    buffer. The programm can now work with the data in the FIFO or   */
/*    not. If the FIFO buffer is full and more data strings are        */
/*    received a buffer overflow occurs and the oldest data will be    */
/*    overwritten with the newest.                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/



/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include <string.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == GLOBAL DATA ==================================================== */
/*                                                                     */
unsigned char gbl_rcintcounter = 0;		    		// Do not change this value, counter variable for UART interrupt
unsigned char gbl_dataavail_type = 0;				// Indicates what kind of data is available: 0=no data available, 1=configuration data, 2=transmit/receive data
unsigned char gbl_receive_buffer[MBF];				// Holds the last received bytes from UART/I2C/SPI
unsigned char gbl_uart_receive_buffer[MBF];			// Holds the actual received byte from UART
unsigned char gbl_spi_receive_buffer[MBF];   		// Holds the actual received byte from SPI
unsigned char gbl_i2c_receive_buffer[MBF];   		// Holds the actual received byte from I2C

unsigned char gbl_radio_output_muxbuffer[10][MBF];	// The output FIFO buffer
unsigned char gbl_radio_output_muxack[10];			// The output FIFO acknowledge buffer
unsigned char gbl_radio_output_muxrec[10];			// The output FIFO receiver buffer
unsigned char gbl_radio_output_locnextfreepack = 0;	// Counter for the output FIFO buffer to identify the next free position, do not change!
unsigned char gbl_radio_output_muxnbr[10];			// In variable char length mode the length of all chars in the FIFO output buffer is stored here
unsigned char gbl_select_interface = 0;				// 0=No interface selected, 1=UART, 2=I2C, 3=SPI
unsigned char gbl_radio_sendack = 0; 				// Flag to indicate whether the user wishes a ACK or not: 0=no ack, 1=ack
unsigned char gbl_radio_recack = 0;					// Flag to indicate whether the transmitter wishes a ACK or not: 0=no ack, 1=ack

volatile unsigned char gbl_INT_Timer0_Counter = 0;	// Do not change this value, counter variable for timer0
volatile unsigned char gbl_INT_Timer0_Timeout = 0;	// Flag for timeout: 0 = No Timeout, 1 = Timeout
volatile unsigned char gbl_INT_Timer2_Counter = 0;	// Counter for the timer2, do not change this value
volatile unsigned char gbl_INT_Timer2_Counter_Max;	// Maximum cycles for the timer2 before the code in the interrupt is executed
volatile unsigned char gbl_INT_Timer1_Counter = 0;	// Do not change this value, counter variable for timer1
volatile unsigned char gbl_INT_Timer1_State   = 0;	// Current state of the Timer1: 0=Nothing,1=RCC-LED,2=ACK-Timeout
volatile unsigned char gbl_INT_Timer1_Timeout = 0;	// Flag for timeout: 0 = No Timeout, 1 = Timeout

/* == FUNCTION INT_UART_ReceiveComplete ============================== */
/*                                                                     */
/*    This function is called by the UART0_RECEIVE_INTERRUPT vector    */
/*    in uart.c. This function receives the characters and stores them */
/*    in a char array.                                                 */
/*    What to do with these bytes depends on the status of  MODESELECT */
/*    Pin. If the Pin is held high the controller is in configuration  */
/*    mode. So it expects 2 characters to form a 16 bit word.          */
/*    If the MODESELECT Pin is held low the controller is in receive/  */
/*    transmit mode.                                                   */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void INT_UART_ReceiveComplete(void)
{
	U_INT16 uart_receive;

	/* 1. Receive the actual character    */
	/* ---------------------------------- */

	uart_receive = uart_getc();

        TCNT2 = 0;                                  // Reset Timer2
        TCCR2 STARTTIMER2;   	                    // Start Timer2


	gbl_select_interface = 1;		// Selected interface is now UART
	/* 2. Check if the value is valid     */
	/* ---------------------------------- */
    if ( uart_receive & UART_NO_DATA )
    	;// No data available from UART
    else
    {
        if ( uart_receive & UART_FRAME_ERROR )
			error_handler(FRAMEERROR);    // Frame Error, no Stop-Bit detected
										  // Send this error to the error handler
        else if ( uart_receive & UART_OVERRUN_ERROR )
			error_handler(OVERRUNERROR);  // Overrun Error, one ore more characters have been dropped
										  // Send this error to the error handler
        else if ( uart_receive & UART_BUFFER_OVERFLOW )
			error_handler(BUFFEROVERFLOW);// Bubber Overflow, the receive buffer was not reading fast enough
										  // Send this error to the error handler

	/* 3. Save character in array         */
	/* ---------------------------------- */
		else			
		{		
			// The received character is valid
               if(gbl_rcintcounter==0)
			{
				gbl_INT_Timer2_Counter = 0;			// Reset Timer2	Counter Variable
				TCNT2 = 0; 							// Reset Timer2 Counter Register
				TIMSK |= (1<<OCIE2);	    		// Enable Interrupt
                TCNT2 = 0;                          // Reset Timer2
                TCCR2 STARTTIMER2;		            // Start Timer2
			}
			else
			{
				TCNT2 = 0; 							// Reset Timer2	
				gbl_INT_Timer2_Counter = 0;			// Reset Timer2

                TCCR2 STARTTIMER2;		            // Start Timer2
			}

			if(!((CTRL_PIN_MDSEL) & (1<<CTRL_IPN_MDSEL)))		
			{
				if(gbl_rcintcounter==0)					// First received byte
				{
					gbl_config_id_target=uart_receive;	// Save first byte as the target ID
				}
				if(gbl_rcintcounter==1)					// Second received byte, Check for Acknowledge
				{
					if(uart_receive==0xAA)				
						gbl_radio_sendack = 1;			
					if(uart_receive==0xF0)
						gbl_radio_sendack = 0;
				}
				if(gbl_rcintcounter>1)
				{
					gbl_uart_receive_buffer[gbl_rcintcounter-2] = uart_receive;	// Copy char into the first array
					gbl_uart_receive_buffer[gbl_rcintcounter-1] = '\0';			// Add a string-end command 	
				}
			}
			else if((CTRL_PIN_MDSEL) & (1<<CTRL_IPN_MDSEL))	
			{
				gbl_uart_receive_buffer[gbl_rcintcounter] = uart_receive;	// Copy char into the first array
				gbl_uart_receive_buffer[gbl_rcintcounter+1] = '\0';			// Add a string-end command 
			}							

        }
	}

	/* 4. Increment the counter           */
	/* ---------------------------------- */
	gbl_rcintcounter++;


	/* 5. Analyze the mode                */
	/* ---------------------------------- */	
		// Do nothing, wait for timer2 interrupt
}	

/* == FUNCTION handle_outputbuffer =================================== */
/*                                                                     */
/*    This function controls the ouput buffer.                         */
/*    All 3 interfaces (UART, I2C and SPI) use this function.          */
/*    The function is only used with radio transmission data strings.  */
/*    The function copies the fully received string into a FIFO buffer */
/*    that is used by other functions in the received_data.c file.     */
/*    The size of the (ouput) FIFO buffer is controlled by the         */
/*    gbl_config_fifooutput parameter. An error is reported when there */
/*    is an buffer overflow.                                           */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char *receive_buffer_p : Pointer to the data string     */
/*    unsigned char len               : Length of the char-array       */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void handle_outputbuffer(unsigned char *receive_buffer_p, unsigned char len)
{


	if(gbl_radio_output_locnextfreepack<gbl_config_fifooutput)
	{
		// Copy received data into the FIFO output buffer
		strncpy(gbl_radio_output_muxbuffer[gbl_radio_output_locnextfreepack],receive_buffer_p,len);
		// Copy information about the acknowledgement
		gbl_radio_output_muxack[gbl_radio_output_locnextfreepack] = gbl_radio_sendack;
		// Copy information about the target device
		gbl_radio_output_muxrec[gbl_radio_output_locnextfreepack] = gbl_config_id_target;
		//Save the number of chars that are stored in the FIFO output buffer
		gbl_radio_output_muxnbr[gbl_radio_output_locnextfreepack] = len;

		gbl_radio_output_locnextfreepack++;

	}
	else if(gbl_radio_output_locnextfreepack>=gbl_config_fifooutput)
	{
		// Overflow!
		error_handler(RADIO_OUTBUFFEROVERFLOW);		// Send error message
		gbl_radio_output_bufferoverflow = 1;		// Set overflow flag
		gbl_radio_output_locnextfreepack=0;

		// Copy received data into the FIFO output buffer
		strncpy(gbl_radio_output_muxbuffer[gbl_radio_output_locnextfreepack],receive_buffer_p,len);
		// Copy information about the acknowledgement
		gbl_radio_output_muxack[gbl_radio_output_locnextfreepack] = gbl_radio_sendack;
		// Copy information about the target device
		gbl_radio_output_muxrec[gbl_radio_output_locnextfreepack] = gbl_config_id_target;
		//Save the number of chars that are stored in the FIFO output buffer
		gbl_radio_output_muxnbr[gbl_radio_output_locnextfreepack] = len;

		gbl_radio_output_locnextfreepack++;
	}
}


/* == FUNCTION TIMER2_COMP_vect ====================================== */
/*                                                                     */
/*    When a interface (UART, SPI or I2C) receives                     */
/*    radio transmission data strings, these strings are stored in a   */
/*    array. If no more characters are received for a certain time the */
/*    timer2 interrupt occurs and the received characters are stored   */
/*    in the FIFO output buffer.                                       */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
ISR(TIMER2_COMP_vect)
{
	
	if(gbl_INT_Timer2_Counter >= gbl_INT_Timer2_Counter_Max)	
	{
		TIMSK &= ~(1<<OCIE2);		// Disable timer2
        TCCR2 &= ~((1<<CS20)|(1<<CS21)|(1<<CS22));// Stop Counter2

		gbl_INT_Timer2_Counter = 0;	// Set back the timer2 counter
		if((CTRL_PIN_MDSEL) & (1<<CTRL_IPN_MDSEL))
		{
			// The MODESELECT-Pin is held high, so the board is in configuration mode
			if(gbl_rcintcounter>=2)                         // Respond on the third interrupt
			{
				strncpy(gbl_receive_buffer,gbl_uart_receive_buffer,2);	// Copy from the first to the second buffer
				gbl_rcintcounter=0;				// Set counter variable back to zero
				rf12_enabletimeout();			// Enable Timeout Timer
				gbl_dataavail_type = 1;			// New configuration data available				
			}
			else
			{
				gbl_rcintcounter=0;	
			}
		}
		else if(!((CTRL_PIN_MDSEL) & (1<<CTRL_IPN_MDSEL)))
		{
			// The MODESELECT-Pin is held low, so the board is in receive/transmit mode
			gbl_radio_output_bufferready = 1;	// FIFO output buffer is busy
			handle_outputbuffer(gbl_uart_receive_buffer,strlen(gbl_uart_receive_buffer));	// Call function to transfer the received value into the output buffer	
			gbl_rcintcounter=0;					// Set counter variable back to zero
			rf12_enabletimeout();				// Enable Timeout Timer
			gbl_dataavail_type = 3;				// New radio transmission data is available, but not fixed length
		}
		gbl_rcintcounter=0;			// Set back the interrupt byte counter
	}
	else
		gbl_INT_Timer2_Counter++;
}


/* == FUNCTION TIMER0_COMP_vect ====================================== */
/*                                                                     */
/*    This interrupt is called on the compare match of timer0.         */
/*    Timer0 is used to create a timeout for the rf12_rxdata().        */ 
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
ISR(TIMER0_COMP_vect)
{
	if(gbl_INT_Timer0_Counter>=2)
	{
        TCCR0 &= ~((1<<CS00)|(1<<CS01)|(1<<CS02));      // Stop Timer0
		TIMSK &= ~(1<<OCIE0);		// Disable timer0
		gbl_INT_Timer0_Timeout=1;	// Set Timeout flag
		gbl_INT_Timer0_Counter=0;	// Set counter value to 0
	}
	else
		gbl_INT_Timer0_Counter++;	// Increase counter value
}


/* == FUNCTION INT0_vect ============================================= */
/*                                                                     */
/*    This interrupt is called at/after:                               */
/*    - Any logic change                                               */ 
/*    - Wake up                                                        */
/*                                                                     */
/*    At any time the interrupt is called the interrupt byte counter   */
/*    is set to zero. This is neccessary because when you switch from  */
/*    radio transmission mode to configuration mode (or vice versa)    */
/*    previous transfered bytes could be stored in the buffer.         */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
ISR(INT0_vect)
{
	gbl_rcintcounter = 0;		// Set the interrupt counter to zero,
	// Do nothing more 
}


/* == FUNCTION TIMER1_COMPA_vect ===================================== */
/*                                                                     */
/*    This interrupt can be used for multiple options.                 */
/*    At first it is used to turn off the Receive-Complete-LED after   */
/*    it is turned on by the rf12_rxdata() function.                   */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
ISR(TIMER1_COMPA_vect)
{
	if(gbl_INT_Timer1_State==1)
	{
		/*    1. LEDs                */
		/* ------------------------- */
		if(gbl_INT_Timer1_Counter==8)
		{
			gbl_INT_Timer1_Counter = 0;			// Reset counter
			set_leds(LED_RECEIVING,LED_OFF); 	// Turn of Receive-Complete-LED
            TCCR1B &= ~ ((1<<CS10)|(1<<CS11)|(1<<CS12));// Stop Counter1
			TIMSK &= ~(1<<OCIE1A);				// Disable timer1 
		}
		gbl_INT_Timer1_Counter++;
	}
	if(gbl_INT_Timer1_State==2)
	{
		/*    2. ACK Timeout         */
		/* ------------------------- */
		if(gbl_INT_Timer1_Counter==3)			// Interrupt after 18ms
		{
			gbl_INT_Timer1_Counter = 0;			// Reset counter
			gbl_INT_Timer1_Timeout = 1;			// Enable Timeout
            TCCR1B &= ~ ((1<<CS10)|(1<<CS11)|(1<<CS12));// Stop Counter1
			TIMSK &= ~(1<<OCIE1A);				// Disable timer1
		}
		gbl_INT_Timer1_Counter++;
	}
}





/** END OF MODULE ******************************************************/
