/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : received_data                Source: received_data.c    */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains functions to receive and transmit data.     */
/*    It possesses 3 functions. One function to transmit data via      */
/*    the RF12 module, one function to receive data via the RF12       */
/*    module and one function to indicate the kind of data that is     */
/*    received by the I2C/SPI/UART-Interface.                          */
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <string.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == GLOBAL DATA ==================================================== */
/*                                                                     */
unsigned char gbl_radio_input_fifobuffer[MBF];				// Second input buffer 
unsigned char gbl_radio_input_muxbuffer[10][MBF];			// The input FIFO buffer
unsigned char gbl_radio_input_bufferoverflow = 0;			// Flag to indicate the overflow status of the in-buffer (0=No overflow, 1=Overflow)
unsigned char gbl_radio_input_charnumber[10];				// In variable char length mode the length of all chars in the FIFO input buffer is stored here

unsigned char gbl_radio_output_fifobuffer[MBF];				// Second output buffer
unsigned char gbl_radio_output_bufferoverflow = 0;			// Flag to indicate the overflow status of the out-buffer (0=No overflow, 1=Overflow)
unsigned char gbl_radio_output_bufferready = 0;				// Flag to indicate whether the programm is ready to receive the next byte (0) or not (1)

volatile unsigned char gbl_radio_input_locnextfreepack = 0;	// Counter for the input FIFO buffer to identify the next free position, do not change!

/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */

/* == FUNCTION radio_transmit_data =================================== */
/*                                                                     */
/*    This function transfers data strings from the external micro-    */
/*    controller to the RFM12 module. During the transmission of data  */
/*    bytes the transmitting-LED is turned on.                         */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char unfixed_char_number : length of the transfered     */
/*                                        string (only needed in var-  */
/*                                        iable char length mode)      */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void radio_transmit_data(unsigned char *data, unsigned char startchar, unsigned char charlength, unsigned char target_id, unsigned char acknowledge)
{	
	unsigned char acknowledge_answer, last_ackresponse, all_good;
	unsigned char count_attempts = 5, i=0;

	if(gbl_config_ledsactive == 0)			// If LEDs are active, turn on the transmitting LED
		set_leds(LED_TRANSMITT,LED_ON);

	if(acknowledge==0)						// No Acknowledge command
        rf12_txdata(data,charlength, startchar, gbl_config_id_own, target_id, acknowledge);		// Transmit the data string

	if(acknowledge==1)						// Acknowledge command
	{
		while(i<count_attempts)		
		{
			rf12_txdata(data,charlength, startchar, gbl_config_id_own, target_id, acknowledge);		// Transmit the data string
			acknowledge_answer = rf12_rxack(startchar);
	
			switch(acknowledge_answer)
			{
				case 0:	// Error occured, try ones more!	
						last_ackresponse = 0;	
						all_good = 0;
						break;
				case 1:	// Timeout, try ones more!
						last_ackresponse = 1; 
						all_good = 0;
						break;
				case 2: // Acknowledge, all okay, exit!
						last_ackresponse = 2;
						all_good = 1;
						i=count_attempts;
						break;
				case 3:	// Not Acknowledge, try ones more!
						last_ackresponse = 3;
						all_good = 0;
						break;
			}
			i++;
		}
		if(all_good==0)
		{
			//Error occured, write error into the error handler
			if(last_ackresponse==0)
				error_handler(TRANSMIT_ERROR);
			if(last_ackresponse==1)
				error_handler(TRANSMIT_TIMEOUT);
			if(last_ackresponse==3)
				error_handler(TRANSMIT_NACK);
		}
		if(all_good==1)
		{	
			 //More than one attempt to send the package?
			 //Write it into the error handler!
			 i = i -(1+count_attempts);

			if(i>1)
				error_handler(TRANSMIT_SOMETRY);
		}
	}
	i=0;
	set_leds(LED_TRANSMITT,LED_OFF);		// Turn off the transmit LED 
}


/* == FUNCTION radio_receive_data ==================================== */
/*                                                                     */
/*    This function calls the rf12_rxdata() and checks if the RFM12    */
/*    module has received any data. If data is available and valid     */
/*    it depends on the status of the pollmode if the data is send to  */
/*    an external microcontroller or not.                              */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void radio_receive_data(void)
{		
	unsigned char rf12_rxdata_errorreport, datastringlen;

	rf12_rxdata_errorreport = rf12_rxdata(gbl_radio_input_fifobuffer,gbl_config_startchar,&datastringlen);
	if(rf12_rxdata_errorreport==1)					// Receive Data, check if data is valid
	{			
		// Data is valid!! 
		if(gbl_radio_recack==1)
			rf12_txack(ACK,gbl_config_startchar);			// Send ACK if the transmitter wishes a response

		CTRL_PORT_RECC |= (1<<CTRL_PNR_RECC);			// Set "received data valid" indicator high				
		
		if((gbl_select_interface==1)&&(gbl_config_pollmode==0))				// Poll mode disabled, data is automatically transfered to the external microcontroller
		{
			uart_putc(gbl_config_id_target);				// Send ID of the target device	
			uart_puts(gbl_radio_input_fifobuffer);			// Send Data
			CTRL_PORT_RECC &= ~(1<<CTRL_PNR_RECC);		// Set "received data valid" indicator low	
		}
		else
		{
			// Poll Mode active, no automatically transfer of data
			cli();		// Disable global interrupts 
			if(gbl_radio_input_locnextfreepack<gbl_config_fifoinput)
			{
				
				// Copy received data into the FIFO input buffer
				strncpy(gbl_radio_input_muxbuffer[gbl_radio_input_locnextfreepack],gbl_radio_input_fifobuffer,datastringlen+1);
				//Save the number of chars that are stored in the FIFO output buffer
				gbl_radio_input_charnumber[gbl_radio_input_locnextfreepack] = datastringlen;
				gbl_radio_input_locnextfreepack++;
			}			
			else if(gbl_radio_input_locnextfreepack>=gbl_config_fifoinput)
			{
				// Overflow!
				error_handler(RADIO_INBUFFEROVERFLOW);		// Send error message
				gbl_radio_input_bufferoverflow = 1;			// Set overflow flag
				gbl_radio_input_locnextfreepack=0;
				strncpy(gbl_radio_input_muxbuffer[gbl_radio_input_locnextfreepack],gbl_radio_input_fifobuffer,datastringlen+1);
				//Save the number of chars that are stored in the FIFO output buffer
				gbl_radio_input_charnumber[gbl_radio_input_locnextfreepack] = datastringlen;
				gbl_radio_input_locnextfreepack++;
			}
			sei();		// Enable global interrupts
		}
		
		}
	else if(rf12_rxdata_errorreport==0)	
	{
		// An error occured, report this error
		error_handler(RADIO_DATANOTVALID);	
	}
	else if(rf12_rxdata_errorreport==2)
	{
		// Timeout, do nothing	
	}
	else if(rf12_rxdata_errorreport==3)
	{
		// Send NACK if the transmitter wishes a response
		if(gbl_radio_recack==1)
			rf12_txack(NACK, gbl_config_startchar);		
	}
}


/* == FUNCTION is_data_available ===================================== */
/*                                                                     */
/*    This function checks if data from the external microcontroller   */
/*    or computer is available in the input buffer.                    */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void is_data_available(void)
{
	U_INT16 config_word;
	unsigned char i,k;
	if(gbl_dataavail_type==0)
	{
		// No data via UART, I2C or SPI is available at the moment
	}
	if(gbl_dataavail_type==1)
	{
		// Configuration data is available 
		config_word = (gbl_receive_buffer[0] << 8) | (gbl_receive_buffer[1]);
		configuration_main(config_word);		// Do something with the received data

		gbl_dataavail_type = 0;					// No more data available		
	}
	if(gbl_dataavail_type==3 && (gbl_radio_output_locnextfreepack!=0))
	{
		// Copy the content of the element 0 to the output buffer
    	strncpy(gbl_radio_output_fifobuffer,gbl_radio_output_muxbuffer[0],gbl_radio_output_muxnbr[0]);

		gbl_dataavail_type = 0;					// No more data available

		gbl_radio_output_locnextfreepack--;		// Decrease the counter
		radio_transmit_data(gbl_radio_output_fifobuffer,gbl_config_startchar,gbl_radio_output_muxnbr[0],gbl_radio_output_muxrec[0],gbl_radio_output_muxack[0]);					// Call the function to transmit the data string
		gbl_radio_output_bufferready = 0;		// FIFO output register is now ready


		// Shift FIFO buffer
		for(i=0;i<gbl_radio_output_locnextfreepack;i++)
		{
            for(k=0;k<=MBF;k++)
				gbl_radio_output_muxbuffer[i][k] = gbl_radio_output_muxbuffer[i+1][k];

			gbl_radio_output_muxnbr[i] = gbl_radio_output_muxnbr[i+1];
			gbl_radio_output_muxack[i] = gbl_radio_output_muxack[i+1]; 
			gbl_radio_output_muxrec[i] = gbl_radio_output_muxrec[i+1]; 
		}

			
		gbl_dataavail_type = 0;	

	}

}




/** END OF MODULE ******************************************************/
