/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : status_register              Source: status_register.c  */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains the status register.                        */
/*    The status register is a part of the configuration, but in order */
/*    to have a better overview it is best to "extract" the status     */
/*    register out of the other configuration functions.               */
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == GLOBAL DATA ==================================================== */
/*                                                                     */

/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */


/* == FUNCTION configuration_status ================================== */
/*                                                                     */
/*    This function gets the actual status of the system and send this */
/*    status to a external microcontroller.                            */
/*                                                                     */
/*    The status register:                                             */
/*                                                                     */
/*      -------------------------------------------------              */
/*    	|15|14|13|12|11|10|09|08|07|06|05|04|03|02|01|00|              */
/*      -------------------------------------------------              */
/*    	| 1| 0| 1| 1|lb|e3|e2|e1|e0|er|fv|fb|fr|fo|fe|fd|              */
/*      -------------------------------------------------              */
/*                                                                     */
/*    fd     : FIFO input buffer data available                        */
/*    fe     : FIFO input buffer empty                                 */
/*    fo     : FIFO input buffer overflow (cleared with status read)   */
/*    fr     : FIFO output buffer ready to receive next byte           */
/*    fb     : FIFO output buffer empty                                */
/*    fv     : FIFO output buffer overflow (cleared with status read)  */
/*    er     : Error occured? (1=Yes, 0=No)                            */
/*    e0-e3  : Error message                                           */
/*    lb     : Low battery flag (only significant if ZLDO active       */
/*             (0 = Battery ok, 1 = Low battery)                       */
/*                                                                     */
/*    Please notice: The status command do not communicate with the    */
/*                   RFM12 module. So, when you want to read the       */
/*                   status, the RFM12 module is NOT busy at this time */
/*                   you can call this function whenever you want.     */
/*                   The use of the fr-Bit is only neccessary when you */
/*                   don't work with the output buffer. At every time  */
/*                   you can write data in the module and it is stored */
/*                   in the FIFO output buffer. The fr-But only        */
/*                   indicates when the data that is send is trans-    */
/*                   mitted.                                           */                                          
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char send_status: 1=send status (via UART, I2C, SPI)    */
/*                               0=don't send status, just update it   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void configuration_status(unsigned char send_status)
{
	unsigned char error_message;
	
	// Get the status of the input and output FIFO
	if(gbl_radio_input_locnextfreepack > 0)				// Is data in the input buffer available?
		gbl_config_status |= (1<<FIFO_IN_DATAAVAIL);
	else
		gbl_config_status &=~(1<<FIFO_IN_DATAAVAIL);

	if(gbl_radio_input_locnextfreepack == 0)			// Is the input buffer empty?
		gbl_config_status |= (1<<FIFO_IN_EMPTY);
	else
		gbl_config_status &=~(1<<FIFO_IN_EMPTY);

	if(gbl_radio_input_bufferoverflow == 1)				// Was there an overflow in the input buffer?
		gbl_config_status |= (1<<FIFO_IN_BUFFOVERFLOW);
	else
		gbl_config_status &=~(1<<FIFO_IN_BUFFOVERFLOW);

	if(gbl_radio_output_bufferready == 0)				// Is the output buffer ready to receive the next byte?
		gbl_config_status |= (1<<FIFO_OUT_READY);	
	else
		gbl_config_status &=~(1<<FIFO_OUT_READY);

	if(gbl_radio_output_locnextfreepack == 0)			// Is the output buffer empty?	
		gbl_config_status |= (1<<FIFO_OUT_EMPTY);
	else
		gbl_config_status &=~(1<<FIFO_OUT_EMPTY);

	if(gbl_radio_output_bufferoverflow == 1)			// Was there an overflow in the output buffer?
		gbl_config_status |= (1<<FIFO_OUT_BUFFOVERFLOW);
	else
		gbl_config_status &=~(1<<FIFO_OUT_BUFFOVERFLOW);

	// Clear the overflow flag in the input and output buffer
	gbl_radio_input_bufferoverflow = 0;			
	gbl_radio_output_bufferoverflow = 0;

	// Handle error status
	error_message = error_handler(DUMMY);		// Call the error handler and receive the last reported error
	
	if(error_message == 0)
		gbl_config_status &= ~(1<<ERROR_STATUS);				
	else 	
		gbl_config_status |= (1<<ERROR_STATUS);	
	
	gbl_config_status |= (error_message<<ERROR_BEGIN_MESSAGE);	// Copy last reported error into status register

	// Low Battery Flag indicator, only used if SC pin of the
	// ZLDO is held low and ZLDO is used in program
	if((ZLDO_PIN_SC & (1<<ZLDO_IPN_SC))==0)
	{
		if(!(ZLDO_PIN_LFB & (1<<ZLDO_IPN_LBF)))
			gbl_config_status |= (1<<LOW_BATTERY_FLAG);
		else 
			gbl_config_status &= ~(1<<LOW_BATTERY_FLAG);
	}
	else
		gbl_config_status &= ~(1<<LOW_BATTERY_FLAG);

	
	// Add one's at bit position 12, 13 and 15
	gbl_config_status |= (1<<12) | (1<<13) | (1<<15);
				
	// Check status
	if(send_status==1)
	{
		if(gbl_select_interface==1)
			uart_putc( (unsigned char)(gbl_config_status>>8));	// Send upper byte
			uart_putc( (unsigned char) gbl_config_status);		// Send lower byte
		if(gbl_select_interface==2)
			;//TODO
		if(gbl_select_interface==3)
			;//TODO
	}
}

/** END OF MODULE ******************************************************/
