/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : uart_bitrate                 Source: uart_bitrate.c     */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains only one function to determine the correct  */
/*    interrupt interval of timer2 when using UART                     */
/*    Read the description of the function below for more information. */ 
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <math.h>
#include <inttypes.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == GLOBAL DATA ==================================================== */
/*                                                                     */

/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */


/* == FUNCTION compute_bitrate ======================================= */
/*                                                                     */
/*    This function computes the interrupt time for the timer2 when    */
/*    variable char length mode is used. After every received byte     */
/*    (e.g. from the UART) the timer2 (re-)starts.                     */
/*    After a certain time the code in the interrupt-routine of the    */
/*    timer2 is called and a new data package of received characters   */
/*    is created. When the UART is used, the time while the programm   */
/*    "waits" for new incomming characters (before the timer2          */
/*    interrupt is executed) is mainly determined by the baudrate.     */
/*    The programm waits exactly that time that a character with a     */
/*    given baudrate needs to be completely received by the uart.      */
/*    By 8N1-UART-Mode one "character" consists of 10 bits.            */
/*                                                                     */
/*    So for example: When baud 19200 is used, one bit needs 52s to   */
/*    be transfered. 10 bits (one package) need 520s. So, after the   */
/*    last fully received character the programm waits 520s for the   */
/*    next character before it stops waiting.                          */
/*    With baud 38400 the programm waits 320s and so on.              */
/*                                                                     */
/*    To make this code fully compliant with any clock source and with */
/*    any clock speed, this function computes this "interrupt time"    */
/*    with a given CPU period you can change in the defines.h          */
/*                                                                     */
/*    To avoid floating point operations all calculations are made     */
/*    with nanoseconds timebase. To compute the interrupt cycles       */
/*    this function needs about 5ms and is only called in the          */
/*    initializing sequence and when the baudrate is changed.          */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    unsigned char : Cycles for the timer2 before the interrupt is    */
/*                    executed.                                        */
/*                                                                     */
/***********************************************************************/
unsigned char compute_bitrate(unsigned char baudrate)
{
	volatile uint32_t baud_bitrate;
//	volatile uint32_t test = 0;
	volatile uint32_t interrupt_cycles=1;

	//First select the baudrate and its duration in nanoseconds to transmit
	//one character (10 bits)
	switch(baudrate)
	{
		case 0:  // 1200 Baud
				 baud_bitrate = 8330000 * 2; 	// Duration to transfer one package (10 bits) at the given baudrate
				 break;
		case 1:  // 2400 Baud
				 baud_bitrate = 4170000 * 2;  
				 break;
		case 2:  // 4800 Baud
				 baud_bitrate = 2080000 * 2;  
				 break;
		case 3:  // 9600 Baud
				 baud_bitrate = 1040000 * 2;  
				 break;
		case 4:  // 14400 Baud
				 baud_bitrate = 700000 * 2;  
				 break;
		case 5:  // 19200 Baud
				 baud_bitrate = 520000 * 2;  
				 break;
		case 6:  // 28800 Baud
				 baud_bitrate = 350000 * 2;  
				 break;
		case 7:  // 38400 Baud
				 baud_bitrate = 270000 * 2;  
				 break;
		case 8:  // 57600 Baud
				 baud_bitrate = 180000 * 2;  
				 break;
		case 9:  // 76800 Baud
				 baud_bitrate = 130000 * 2;  
				 break; 
		case 10: // 115200 Baud
				 baud_bitrate = 90000 * 2;  
				 break;
		case 11: // 230400 Baud
				 baud_bitrate = 50000 * 2;  
				 break;
		case 12: // 250000 Baud
				 baud_bitrate = 40000 * 2;  
				 break;
	}


	OCR2 = 1;	// Set the OCR2 register to its minimum value

	//Compute the value of the OCR2 register and the cycles for the timer2
	//before the code in the interrupt is executed
	while(baud_bitrate > ((F_PERIOD*OCR2)*interrupt_cycles))
	{
		if(OCR2 < 255)
			OCR2++;
		else if(OCR2 == 0xFF)
			interrupt_cycles++;
	}

	//Maybe the time is a bit to long, determine the proper time
	if(interrupt_cycles > 1)
	{
		while((baud_bitrate+1000) < (F_PERIOD*OCR2*interrupt_cycles))
			OCR2--;
	}
	return ((unsigned char) interrupt_cycles);
}

/** END OF MODULE ******************************************************/
