/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Funkmodul                    Autor : Florian Scherb     */
/*    Module : rf12                         Source: rf12.c             */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains all functions to communicate directly with  */
/*    the HOPE RF - RF12 module.                                       */ 
/*                                                                     */
/*                                                                     */
/***********************************************************************/


/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "../headers/defines.h"
#include "../headers/global.h"
#include "../uart/uart.h"
#include <util/crc16.h>

/* == GLOBAL DATA ==================================================== */
/*                                                                     */
volatile unsigned char gbl_RX_buffer = 0;			// Flag for the RX buffer: 0 = buffer empty, 1 = buffer full	
unsigned char gbl_rxdata_errormessage = 0;		 	// This flag indicates in the rf12_rxdata function if a error message
													// is reported (flag=1) or not (flag=0)

/* == DEFINITION OF LOCAL SYNONYMS =================================== */
/*                                                                     */

/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */
void rf12_enabletimeout(void);
void rf12_disabletimeout(void);
unsigned char rf12_readytimeout(void);

/* == FUNCTION rf12_trans ============================================ */
/*                                                                     */
/*    This function transfers a word (16 bits, 2 bytes) to the module. */
/*    It is completeley independent from any hardware interfaces like  */
/*    Hardware-SPI. You can use any available I/O-Pin of your Micro-   */
/*    controller.                                                      */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned short wert : Transfered word               			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    unsigned short werti: Received word							   */
/*                                                                     */
/***********************************************************************/
unsigned short rf12_trans(unsigned short wert)
{	unsigned short werti = 0;
	unsigned char i;

	cbi(RF_PORT, CS);				// Select RFM12
	for (i=0; i<16; i++)			// Iterate 16 times (for 16 bits)
	{	
		if (wert&0x8000)
			sbi(RF_PORT, SDI);
		else
			cbi(RF_PORT, SDI);
		werti<<=1;

		if (RF_PIN&(1<<SDO))
			werti|=1;
		sbi(RF_PORT, SCK);
		wert<<=1;
		_delay_us(0.2);
		cbi(RF_PORT, SCK);
	}
	sbi(RF_PORT, CS);				// Deselect RFM12
	cbi(RF_PORT, SDI);				// SDI to low
	return werti;					// Return received value
}
/* == FUNCTION rf12_init ============================================= */
/*                                                                     */
/*    This function initializes	the RFM12 module.         			   */
/*    The function MUSST be called before the module is used at the    */
/*    first time.                                                      */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_init(void)
{
	RF_DDR=(1<<SDI)|(1<<SCK)|(1<<CS);
	RF_PORT=(1<<CS);

#ifndef DEBUG
		_delay_ms(100);			// wait until POR done
#endif

	rf12_trans(0x80D7);			//Config. Sett. : EL=1, EF=1, 433MHz Band, 12.0 pF 	
	rf12_trans(0x8208);			//Power Manag.  : ER=0, EBB=0, ET=0, ES=0, EX=1, EB=0, EW=0, DC=0
	rf12_trans(0xA640);			//Frequ. Setting: A140=434MHz
	rf12_trans(0xC647);			//Data Rate     : 4,8kbps
	rf12_trans(0x94A0);			//Receiver Contr: VDI output enable, Fast response, 134KHz Bandwidth, LNA gain 0 dbm, RSSI -103 dBm
	rf12_trans(0xC2AC); 		//Data Filter   : CR=1 fast mode, S=0 digital filter, DQD Threshold = 4
	rf12_trans(0xCA81);			//FIFO/RST mode : FIFO 13, SYNC, FF=0, DR=0 no highly sensitive RESET mode
	rf12_trans(0xC4E7);			//AFC Command   : Autom. Op = 11, Frequ. Offset = 10, ST=0, FI=1, OE=1, EN=1 
	rf12_trans(0x9850);			//TX Control    : MP=0,9810=30kHz,Maximum output Power
	rf12_trans(0xE000);			//Wakeuptimer   : Function not used
	rf12_trans(0xC800);			//Low Duty Cycle: Function not used
	rf12_trans(0xC000);			//LBD, Prescaler: Function not used 

}


/* == FUNCTION rf12_sleep ============================================ */
/*                                                                     */
/*    This function disables all internal functions, so the module is  */
/*    in a standby-state. The function can be called at any time.      */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_sleep(void)
{
	rf12_trans(0x8201);			// Disable all interal functions
}


/* == FUNCTION rf12_setclk =========================================== */
/*                                                                     */
/*    This function sets the frequency for the clock output pin.       */
/*    The frequency can vary between 1MHz - 10MHz in the following     */
/*    steps:                                                           */
/*    1MHz, 1,25MHz, 1,66MHz, 2MHz, 2,5MHz, 3,33MHz, 5MHz, 10MHz       */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_setclk(unsigned char clkfrequency)
{
	switch(clkfrequency)
	{
		case 0:	rf12_trans(0xC0E0);		// 10,00MHz Clock
				break;
		case 1:	rf12_trans(0xC0C0);		//  5,00MHz Clock
				break;
		case 2:	rf12_trans(0xC0A0);		//  3,33MHz Clock
				break;
		case 3:	rf12_trans(0xC080);		//  2,50MHz Clock
				break;
		case 4:	rf12_trans(0xC060);		//  2,00MHz Clock
				break;
		case 5:	rf12_trans(0xC040);		//  1,66MHz Clock
				break;
		case 6:	rf12_trans(0xC020);		//  1,25MHz Clock
				break;
		case 7:	rf12_trans(0xC000);		//  1,00MHz Clock
				break;
		default:						// Wrong value, do nothing
				break;
	}
}


/* == FUNCTION rf12_wakeup =========================================== */
/*                                                                     */
/*    This function reactivate the module after it was send in the     */
/*    sleep mode with rf12_sleep().                                    */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                              			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_wakeup(void)
{
	rf12_trans(0x80D7);			//Config. Sett. : EL=1, EF=1, 433MHz Band, 12.0 pF 	
	rf12_trans(0x8208);			//Power Manag.  : ER=0, EBB=0, ET=0, ES=0, EX=1, EB=0, EW=0, DC=0
}

/* == FUNCTION rf12_setchannel ======================================= */
/*                                                                     */
/*    The tranceiver frequency is computed with the following equation */
/*    f0 = 10 * (43+F/4000) [MHz]    96 >= F >= 3903                   */
/*    Frequency range: f0 = 430,24 MHz ... 439,75 MHz, df0 = 2,5KHz    */
/*    Due to the fact that intersections in the frequency ranges are   */
/*    possible the stepwidth is from df0 = 2,5KHz to df0* = 250KHz     */
/*    So you can choose between 39 channels.                           */
/*                                                                     */
/*    Channel  0 = 430,25 MHz                                          */
/*    Channel  1 = 430,50 MHz                                          */
/*    Channel  2 = 430,75 MHz                                          */
/*    ...                                                              */
/*    Channel 38 = 439,75 MHz                                          */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char channel : Channel number 0...38        			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_setchannel(unsigned char channel)
{
	U_INT16 F;
	
	if (channel>38) 
		channel = 38; 					// Maximum Channelnumber 38

	F = 96+((U_INT16)channel*100);		// Compute F from the selected channel
	rf12_trans (0xA000|F);				// Set frequency

}


/* == FUNCTION rf12_setbaudrate ====================================== */
/*                                                                     */
/*    This function sets the baudrate for transfer and receive modes.  */
/*                                                                     */
/*    Transfer rate     Register setting      decimal value            */
/*                                                                     */
/*	  1200 kbit/s      => cs=1 R=0x23 (A3)		1                      */
/*	  2400 kbit/s      => cs=1 R=0x11 (91)		2                      */
/*	  4800 kbit/s      => cs=0 R=0x47			3                      */
/*	  9600 kbit/s      => cs=0 R=0x22			4                      */
/*	 19200 kbit/s      => cs=0 R=0x11 			5                      */
/*	 38400 kbit/s      => cs=0 R=0x08			6                      */
/*	 57600 kbit/s      => cs=0 R=0x05			7                      */
/*	115200 kbit/s      => cs=0 R=0x02			8                      */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    BAUDRATE baud : Baudrate (default 9600 Baud)        			   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_setbaudrate(BAUDRATE baud)
{
	switch (baud)
	{
		case BAUD1200	:   rf12_trans(0xC6A3);		// 1200 Baud
							break;
		case BAUD2400	:   rf12_trans(0xC691);		// 2400 Baud
							break;
		case BAUD4800	:   rf12_trans(0xC647);		// 4800 Baud
							break;
		case BAUD9600	:   rf12_trans(0xC622);		// 9600 Baud
							break;
		case BAUD19200	: 	rf12_trans(0xC611);		// 19200 Baud
							break;
		case BAUD38400	:   rf12_trans(0xC604);		// 38400 Baud
							break;
		case BAUD57600  :   rf12_trans(0xC605);		// 57600 Baud
							break;
		case BAUD115200 :   rf12_trans(0xC602);		// 115200 Baud
							break;							 
		default	     	:  	rf12_trans(0xC622);		// 9600 Baud
							break;	
	}	
}	

/* == FUNCTION rf12_setpower ========================================= */
/*                                                                     */
/*    This function sets the power on the antenna output               */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char power : minimum power = 0, maximum power = 7	   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_setpower(unsigned char power)
{	
	unsigned char val;	

	val = 7-power;
	rf12_trans(0x9850 | val);	//!mp,9810=30kHz

}


/* == FUNCTION rf12_setreccontrol ==================================== */
/*                                                                     */
/*    This function contains settings for the bandwidth, the LNA gain  */
/*    and the RSSI.                                                    */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    U_INT16 bandwidth   : Bandwidth in kHz, possible values:   	   */
/*                          400kHz, 340kHz, 270kHz, 200kHz             */
/*                          134kHz,  67kHz                             */
/*    signed char gain    : LNA gain relative to maximum, possible val.*/
/*                          0dB, -6dB, -14dB, -20dB                    */
/*    signed char rssi_th.: RSSI detector threshold, possible values:  */
/*                          -103dBm, -97dBm, -91dBm, -85dBm,           */
/*                           -79dBm, -73dBm, -67dBm, -61dBm            */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_setreccontrol(U_INT16 bandwidth, signed char gain, signed char rssi_threshold)
{
	U_INT16 wert;
	U_INT8 bw,lna, rssi;

	switch(bandwidth)
	{
		case 400: bw = 1;
		case 340: bw = 2;
		case 270: bw = 3;
		case 200: bw = 4;
		case 134: bw = 5;
		case  67: bw = 6;
		default:  bw = 5;
	}	

	switch(gain)
	{
		case   0:  lna = 0;
		case  -6:  lna = 1;
		case -14:  lna = 2;
		case -20:  lna = 3;
		default:   lna = 0;
	}

	switch(rssi_threshold)
	{
		case  -103: rssi = 0;
		case  -97:  rssi = 1;
		case  -91:  rssi = 2;
		case  -85:  rssi = 3;
		case  -79:  rssi = 4;
		case  -73:  rssi = 5;
		case  -67:  rssi = 6;
		case  -61:  rssi = 7;
		default:    rssi = 0;
	}

	wert = 0x9400;				// Kommando, VDI, Fast
	wert = wert | (bw<<5);		// BW einmaskieren
	wert = wert | (lna<<3);		// LNA Gain einmaskieren
	wert = wert | rssi;			// RSSI Threshold einmaskieren
	rf12_trans(wert);

}


/* == FUNCTION rf12_setdqdreport ===================================== */
/*                                                                     */
/*    This function regulates the DQD report "good signal quality".    */
/*    This threshold parameter should be less than 4 in the case when  */
/*    the bit-rate is close to the deviation.                          */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char dqd:  0 <= dqd <= 7    (default 3)                 */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_setdqdreport(unsigned char dqd)
{
	U_INT16 wert;

	wert = 0xC288;				// AL=1, Slow Mode, Digital Filter
	wert = wert | dqd;			
	rf12_trans(wert); 

}


/* == FUNCTION rf12_ready ============================================ */
/*                                                                     */
/*    This function checks whether the module is ready to receive data */
/*    via the SPI-Interface or not. While the SDO-Pin is low, the      */
/*    module is not ready. When the SDO-Pin is high the next data      */
/*    package can be transfered                                        */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_ready(void)
{	
	cbi(RF_PORT, CS);
#ifndef DEBUG
	while (!(RF_PIN&(1<<SDO))); // wait until FIFO ready
#endif
	sbi(RF_PORT, CS);
}


/* == FUNCTION rf12_txdata =========================================== */
/*                                                                     */
/*    This function transfers a datastring to the RFM12 module which   */
/*    should be received by other modules.                             */
/*                                                                     */
/*    DATA STRING LENGTH:                                              */
/*    The length of the data string has 'practically' no maximum.      */
/*    You can easily transfer 40 characters or more in one string.     */
/*    But it is neccessary that the receiver module knows how many     */
/*    characters are expected. So it is recommended that all data      */
/*    strings are of the same length. To make this code more flexible  */
/*    this string length is variable and is not fixed in this code!!   */
/*                                                                     */
/*    START BYTE:                                                      */
/*    In order to make the transfer of data more reliable a start byte */
/*    is used. This startbyte is variable. You can choose any ASCII    */
/*    character. The receiver only accept data strings which           */
/*    first byte is this startbyte.                                    */
/*                                                                     */
/*    CHECKSUM:                                                        */
/*    Every 5th transfered byte is a so called 'Checksum byte'. It     */
/*    makes the transfer of data more reliable. The checksum byte      */
/*    holds the integer sum of all 4 ASCII characters that were send   */
/*    before the checksum byte.                                        */
/*    The receiver only accepts data which checksum bytes are correct. */  
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char *data  : data string (char array)                  */
/*    unsigned char number : number of data bytes to tranfer           */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_txdata(unsigned char *data, unsigned char number, unsigned char startchar, unsigned char own_id, unsigned char target_id, unsigned char acknowledge)
{	
	unsigned char i, k, checksum, id;
	U_INT16 checksum16;

	/*    1. Initialize the TX-sequence    */
	/* ----------------------------------- */
	rf12_trans(0x8208);			// Turn on the crystal oszillator 
	//_delay_ms(1);				// The module need some time
	rf12_trans(0x0000);			// Receive status
	rf12_trans(0x0000);			// Receive status again
	rf12_trans(0x8238);			// Transmitter (TX) enable
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8AA);			// Send initial write command
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8AA);			// Send initial write command
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8AA);			// Send initial write command
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB82D);			// FIFO
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8D4);			// FIFO
		
	checksum = 0;				// Set checksum to zero

	/*     2. Send the start character     */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0xB800|(startchar));				// Send startchar
	checksum += startchar;

	/*        3. Send the target ID        */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0xB800|(target_id));				// Send target ID
	checksum += target_id;						// Compute checksum
	/*   4. Send the own ID and ACK        */
	/* ----------------------------------- */
	rf12_ready();

	id = own_id;								// Copy own ID			
	if(acknowledge==1)							
		id |= 0x80;								// If yes set the MSB to 1

	rf12_trans(0xB800|(id));					// Send own ID (plus maybe the acknowledge-MSB)
	checksum += id;								// Compute checksum
	/*       5. Send the datalength        */
	/* ----------------------------------- */
	rf12_ready();								
	rf12_trans(0xB800|(number));				// Send amount of chars
	checksum += number;

	/*       6. Send the first checksum    */
	/* ----------------------------------- */
	checksum = _crc_ibutton_update(CRC8,checksum);
	rf12_ready();
	rf12_trans(0xB800|(checksum));				// Send first checksum

	/*             7. Send data            */
	/* ----------------------------------- */
	k=0;
	checksum16=0;
	for (i=0; i<number; i++)	
	{		
		rf12_ready();					// Check whether the module is ready or not
		rf12_trans(0xB800|(*data));		// Now send Data bits
		checksum16 += *data;				// Calculate Checksum
		data++;
	} 
	/*     8. Send 16 bit CRC              */
	/* ----------------------------------- */
	rf12_ready();
	checksum16 = _crc16_update(CRC16,checksum16);
	rf12_trans(0xB800|((unsigned char)checksum16));
	rf12_ready();
	rf12_trans(0xB800|((unsigned char)(checksum16>>8)));
	rf12_ready();


	/*       9. Send two dummy bytes       */
	/* ----------------------------------- */
	rf12_trans(0xB800|(0xAA));			// 2 Dummybytes. They are neccessary to transfer the last
	rf12_ready();						// important byte (in this case the checksum-byte correctly
	rf12_trans(0xB800|(0xAA));			// I don't understand why, but without the dummies it won't work!!

	/*   10. Disable transmitting mode     */
	/* ----------------------------------- */
	rf12_ready();						// Check whether the module is ready or not
	rf12_trans(0x8208);					// Transmitter (TX) disable
}



/* == FUNCTION rf12_txack ============================================ */
/*                                                                     */
/*    This function transfers the acknowledge / not acknowledge if it  */
/*    is expected by a other transmitter.                              */
/*                                                                     */
/*    The package has got the following framework:                     */
/*                                                                     */
/*      -------------------              							   */
/*    	|st|it|io|an|dm|dm|              							   */
/*      -------------------                                            */
/*                                                                     */
/*    st: Startbyte                                                    */
/*    it: ID target                                                    */
/*    io: ID own                                                       */
/*    an: Acknowledge / Not Acknowledge                                */
/*    dm: Dummy byte                                                   */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char state : 0=Send ACK, 1=Send NACK                    */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_txack(unsigned char state, unsigned char startchar)
{	
	/*    1. Initialize the TX-sequence    */
	/* ----------------------------------- */
	rf12_trans(0x8208);			// Turn on the crystal oszillator 
	//_delay_ms(1);				// The module need some time
	rf12_trans(0x0000);			// Receive status
	rf12_trans(0x0000);			// Receive status again
	rf12_trans(0x8238);			// Transmitter (TX) enable
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8AA);			// Send initial write command
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8AA);			// Send initial write command
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8AA);			// Send initial write command
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB82D);			// FIFO
	rf12_ready();				// Check whether the module is ready or not
	rf12_trans(0xB8D4);			// FIFO
		
	/*   2. Send the start character       */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0xB800|(startchar));

	/*   3. Send the target ID             */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0xB800|(gbl_config_id_target));

	/*   4. Send the own ID                */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0xB800|(gbl_config_id_own));

	/*   5. Send the ACK / NACK            */
	/* ----------------------------------- */
	rf12_ready();
	if(state==ACK)
		rf12_trans(0xB800|0xAA);		// Send ACK
	if(state==NACK)
		rf12_trans(0xB800|0x0F);		// Send NACK
				
	/*       9. Send two dummy bytes       */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0xB800|(0xAA));			// 2 Dummybytes. They are neccessary to transfer the last
	rf12_ready();						// important byte (in this case the checksum-byte correctly
	rf12_trans(0xB800|(0xAA));			// I don't understand why, but without the dummies it won't work!!

	/*   10. Disable transmitting mode     */
	/* ----------------------------------- */
	rf12_ready();						// Check whether the module is ready or not
	rf12_trans(0x8208);					// Transmitter (TX) disable
}



/* == FUNCTION rf12_rxack ============================================ */
/*                                                                     */
/*    This function should receive a expected ACK/NACK from the target */
/*    device. This function must be called after a data package is     */
/*    transfered and a acknowledge is needed by the receiver.          */
/*                                                                     */
/*    There are 4 possibilities:                                       */
/*                                                                     */
/*    ACKNOWLEDGE:                                                     */
/*    The target device sends a ACK in the given time window and       */
/*    it is fully received by this device.                             */
/*                                                                     */
/*    NOT ACKNOWLEDGE:                                                 */
/*    The target device sends a NACK in the given time window and      */
/*    it is fully received by this device.                             */
/*                                                                     */
/*    NO RESPONSE:                                                     */
/*    No data is received in the given time window. After that time    */
/*    the function will not wait any longer for any data.              */
/*                                                                     */
/*    DATA RUBBISH:                                                    */
/*    Maybe it is possible that some bytes aren't okay. For example    */
/*    the startbyte or the bytes for the IDs. In this case the         */
/*    function returns and will not receive any further data packages. */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char startchar  : data string (char array)              */
/*    unsigned char number : number of data bytes to tranfer           */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    unsigned char        : 0 = Error occured, just jump out		   */
/*                         : 1 = All ok!                               */
/*                                                                     */
/***********************************************************************/
unsigned char rf12_rxack(unsigned char startchar)
{	
	volatile unsigned char received_id, acknack, return_value;

	/*    1. Initialize the RX-sequence    */
	/* ----------------------------------- */
	rf12_trans(0x82C8);					// RX on
	rf12_trans(0xCA81);					// set FIFO mode
	rf12_trans(0xCA83);					// enable FIFO
	rf12_trans(0x0000);					// Read Status 

	if (rf12_acktimeout() == 0)
		return 1;

	/*  2. Turn on Receive-Complete LED    */
	/* ----------------------------------- */

	if (gbl_config_ledsactive == 0)
	{
		set_leds(LED_RECEIVING,LED_ON);
		gbl_INT_Timer1_State = 1;
		TCNT1 = 0;
        TCCR1B |= (1<<CS12);        // Start Timer1 Prescaler 8
		TIMSK |= (1<<OCIE1A);	    // Enable Timer1 Interrupt				
	}
	else if (gbl_config_ledsactive == 1)
		set_leds(LED_RECEIVING,LED_OFF);

	/*  3. Read startchar and compare it   */
	/* ----------------------------------- */
	if(startchar!=(U_INT8)rf12_trans(0xB000))	// Check if data is valid, else return
	{
		rf12_trans(0x8208); // RX-mode off
	    return 0;
	}

	/*  4. Read the own and the target ID  */
	/* ----------------------------------- */
	rf12_ready();	
	received_id=(U_INT8)rf12_trans(0xB000);
	if((received_id!=gbl_config_id_own)&&(received_id!=0xAA))
	{
		// Wrong ID! Not the own ID and not the global ID, exit!
		rf12_trans(0x8208); // RX-mode off
	    return 0;	
	}
	rf12_ready();	

	received_id=(U_INT8)rf12_trans(0xB000);		// Save the ID of the target device
	if(received_id != gbl_config_id_target)
	{
		rf12_trans(0x8208); // RX-mode off
		return 0;										
	}

	/*  5. Check the acknowledge           */
	/* ----------------------------------- */
	rf12_ready();
	rf12_ready();

	acknack=(U_INT8)rf12_trans(0xB000);	

	if(acknack==0xAA)
		return_value = 2;
	else if(acknack==0xF0)
		return_value = 3;
	else
		return_value = 0;

	/*   6. Read dummybytes                */
	/* ----------------------------------- */
	rf12_ready();
	rf12_ready();
	rf12_trans(0xB000);							// Value of Dummies doesn't matter						
	rf12_ready();
	rf12_ready();
	rf12_trans(0xB000);

	/*   7. Cancel RX-mode                 */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0x8208);			

	/*      13. All ok, return 1           */
	/* ----------------------------------- */
	return return_value;
}



/* == FUNCTION rf12_rxdata =========================================== */
/*                                                                     */
/*    This function receives a datastring from the RFM12 module.       */
/*                                                                     */
/*    DATA STRING LENGTH:                                              */
/*    The length of the data string has 'practically' no maximum.      */
/*    You can easily receive 40 characters or more in one string.      */
/*    But it is neccessary that the receiver module knows how many     */
/*    characters are expected. So it is recommended that all data      */
/*    strings are of the same length. To make this code more flexible  */
/*    this string length is variable and is not fixed in this code!!   */
/*                                                                     */
/*    START BYTE:                                                      */
/*    In order to make the transfer of data more reliable a start byte */
/*    is used. This startbyte is variable. You can choose any ASCII    */
/*    character. The receiver only accept data strings which           */
/*    first byte is this startbyte.                                    */
/*                                                                     */
/*    CHECKSUM:                                                        */
/*    Every 5th transfered byte is a so called 'Checksum byte'. It     */
/*    makes the transfer of data more reliable. The checksum byte      */
/*    holds the integer sum of all 4 ASCII characters that were send   */
/*    before the checksum byte.                                        */
/*    The receiver only accepts data which checksum bytes are correct. */ 
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    unsigned char *data  : data string (char array)                  */
/*    unsigned char number : number of data bytes to tranfer           */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    unsigned char        : 0 = Error occured, just jump out		   */
/*                         : 1 = All ok!                               */
/*                         : 2 = Timeout (when transmitting data)      */
/*                         : 3 = Error occured, send NACK              */
/*                                                                     */
/***********************************************************************/
unsigned char rf12_rxdata(unsigned char *data, unsigned char startchar, unsigned char *datastringlen)
{	
	volatile unsigned char i,k,checksum,received_id, number;
	volatile unsigned char checksum16_lowbyte, checksum16_highbyte;
	U_INT16 checksum16, checksum16_rec;

	/*    1. Initialize the RX-sequence    */
	/* ----------------------------------- */
	rf12_trans(0x82C8);					// RX on
	rf12_trans(0xCA81);					// set FIFO mode
	rf12_trans(0xCA83);					// enable FIFO
	rf12_trans(0x0000);					// Read Status 

	checksum = 0;
  
	if (rf12_readytimeout()	== 0)
		return 2;

	// During the following code sections no data in 
	// the FIFO output buffer can be transmitted

	/*  2. Turn on Receive-Complete LED    */
	/* ----------------------------------- */

	if (gbl_config_ledsactive == 0)
	{
		set_leds(LED_RECEIVING,LED_ON);
		gbl_INT_Timer1_State = 1;
		TCNT1 = 0;
        TCCR1B |= (1<<CS12);                    // Start Timer1 Prescaler 8
		TIMSK |= (1<<OCIE1A);					// Enable timer1 interrupt
	}
	else if (gbl_config_ledsactive == 1)
		set_leds(LED_RECEIVING,LED_OFF);

	/*  3. Read startchar and compare it   */
	/* ----------------------------------- */
	rf12_ready();
	if(startchar!=(U_INT8)rf12_trans(0xB000))	// Check if data is valid, else return
	{
		rf12_trans(0x8208); 					// RX-mode off
	    return 0;								// Jump out, send no NACK
	}
	checksum += startchar;

	/*  4. Read the own and the target ID  */
	/* ----------------------------------- */
	rf12_ready();	
	received_id=(U_INT8)rf12_trans(0xB000);
        if((received_id!=gbl_config_id_own)&&(received_id!=0x55))
	{
		// Wrong ID! Not the own ID and not the global ID, exit!
		rf12_trans(0x8208); 					// RX-mode off
	    return 0;								// Jump out, send no NACK
	}
	checksum += received_id;
	rf12_ready();	
	rf12_ready();
	gbl_config_id_target=(U_INT8)rf12_trans(0xB000);	// Save the ID of the target device

	if(gbl_config_id_target & 0x80)
		gbl_radio_recack = 1;							// The transmitter wishes a ACK
	else
		gbl_radio_recack = 0;							// The transmitter wishes no ACK

	checksum += gbl_config_id_target;

	gbl_config_id_target &= ~0x80;						// Delete Acknowledge-MSB

	/*  5. Check the data length           */
	/* ----------------------------------- */
	rf12_ready();
	rf12_ready();
	number = rf12_trans(0xB000);
	*datastringlen = number;
	checksum += number;

	/*  6. Check the first checksum        */
	/* ----------------------------------- */
	rf12_ready();	
	rf12_ready();
	checksum = _crc_ibutton_update(CRC8,checksum);
	if(checksum!=(U_INT8)rf12_trans(0xB000))
	{	
		rf12_trans(0x8208); 					// RX-mode off
		return 0;								// Jump out, send no NACK
	}		

	/* 7. Read Databytes and Checkbytes    */
	/* ----------------------------------- */
	k=0;
	checksum16=0;  
	rf12_ready();
	for (i=0; i<number; i++)
	{
		if (gbl_INT_Timer0_Timeout == 0)	// No Timeout, read next byte
		{
			rf12_ready();
			*data=rf12_trans(0xB000);		// Read next byte
			checksum16 += *data;			// Calculate checksum
			data++;							// Increase address of data pointer
			rf12_ready();
		}
		else								// Timeout! Stop reading!
		{
			return 0;						// Jump out, send no NACK
		}
	}

	/*   8. Read last byte (checkbyte)     */
	/* ----------------------------------- */
	rf12_ready();
	checksum16_lowbyte = (U_INT8)rf12_trans(0xB000);	// Read low byte of 16 Bit CRC
	rf12_ready();
	rf12_ready();
	checksum16_highbyte = (U_INT8)rf12_trans(0xB000);	// Read high byte of 16 Bit CRC

	checksum16_rec = (checksum16_highbyte<<8) | (checksum16_lowbyte);

	if(checksum16_rec!=_crc16_update(CRC16,checksum16))	// Check if data is valid, else return
	{
		rf12_trans(0x8208); 			// RX-mode off
		if(gbl_radio_recack==1)
			return 3;					// Jump out, send NACK
		else
			return 0;					// Jump out, send no NACK
	}

	/*        9. Read dummybytes           */
	/* ----------------------------------- */
	rf12_ready();
	rf12_ready();
	rf12_trans(0xB000);							// Value of Dummies doesn't matter						
	rf12_ready();
	rf12_ready();
	rf12_trans(0xB000);

	/*       10. Cancel RX-mode            */
	/* ----------------------------------- */
	rf12_ready();
	rf12_trans(0x8208);			

	/*  11. Add Null-Byte to datastring    */
	/* ----------------------------------- */
	*data = '\0';

	/* 12. Data-String valid, set strobe   */
	/* ----------------------------------- */
	DDRD  |=  (1<<PD6);
	PORTD |=  (1<<PD6);
	_delay_us(30);				
	PORTD &=  ~(1<<PD6);

	/*      13. All ok, return 1           */
	/* ----------------------------------- */
	return 1;
}


/* == FUNCTION rf12_readytimeout ===================================== */
/*                                                                     */
/*    This function behaves like the rf12_ready()-Function. But unlike */
/*    rf12_ready() this function provides a timeout function.          */
/*    When a character receives in the input buffer of the UART, SPI   */
/*    or I2C timer0 is activated and if the RFM12 module don't receive */
/*    any characters the programm skip out of the rf12_rxdata()        */
/*    function and handle other tasks.                                 */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
unsigned char rf12_readytimeout(void)
{	
	cbi(RF_PORT, CS);
	while (!(RF_PIN&(1<<SDO)))
	{
		if(gbl_INT_Timer0_Timeout==1) 	    // True if the timer0 interrupt is called the third time
		{
			gbl_INT_Timer0_Timeout = 0;
			rf12_trans(0x8208);	   			// Leave RX-Mode
			return 0;
		}

	} // wait until FIFO ready
	sbi(RF_PORT, CS);
	rf12_disabletimeout();			        // Disable timer0
	return 1;
}


/* == FUNCTION rf12_acktimeout ======================================= */
/*                                                                     */
/*    This function includes a timer timeout for the acknowledgement.  */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
unsigned char rf12_acktimeout(void)
{	
	gbl_INT_Timer1_State = 2;
	TCNT1 = 0;
    TCCR1B |= (1<<CS12);                    // Start Timer1 Prescaler 8
	TIMSK |= (1<<OCIE1A);	                // Enable Timer1 Interrupt			
	cbi(RF_PORT, CS);
	while (!(RF_PIN&(1<<SDO)))
	{
		if(gbl_INT_Timer1_Timeout==1) 	    // True if the timer0 interrupt is called the third time
		{
			gbl_INT_Timer1_Timeout = 0;
            TCCR1B &= ~ ((1<<CS10)|(1<<CS11)|(1<<CS12));// Stop Counter1
			TIMSK &= ~(1<<OCIE1A);			// Disable timer1 interrupt
			rf12_trans(0x8208);	   			// Leave RX-Mode
			return 0;
		}

	}
	sbi(RF_PORT, CS);
	gbl_INT_Timer1_Timeout = 0;				// No Timeout
    TCCR1B &= ~ ((1<<CS10)|(1<<CS11)|(1<<CS12));// Stop Counter1
	TIMSK &= ~(1<<OCIE1A);					// Disable timer1 interrupt
	return 1;
}


/* == FUNCTION rf12_enabletimeout ==================================== */
/*                                                                     */
/*    This function enables the timeout.                               */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_enabletimeout(void)
{
	gbl_INT_Timer0_Timeout = 0;	// No Timeout at startup
	OCR0 = 255;					// Compare Value
    TCNT0 = 0;                  // Reset Counterregister    
    TCCR0 |= (1<<CS01);         // Prescaler 8              
	TIMSK |= (1<<OCIE0);		// Enable timer0 interrupt
}


/* == FUNCTION rf12_disabletimeout =================================== */
/*                                                                     */
/*    This function disables the timeout.                              */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none                  										   */
/*                                                                     */
/***********************************************************************/
void rf12_disabletimeout(void)
{
    TCCR0 &= ~((1<<CS00)|(1<<CS01)|(1<<CS02));// Stop Timer0
	TIMSK &= ~(1<<OCIE0);		// Disable timer0 interrupt
	gbl_INT_Timer0_Timeout = 0;
}


/** END OF MODULE ******************************************************/





