/**** IDENTIFICATION ***************************************************/
/*                                                                     */
/*    Project: Reflowofen                    Autor : Florian Scherb    */
/*    Module : uart                          Source: uart.c            */
/*                                                                     */
/*                                                                     */
/* == FUNCTIONAL DESCRIPTION OF MODULE =============================== */
/*                                                                     */
/*    This module contains some extended uart functions since they     */
/*    are needed to simplify the communication between the AVR and     */
/*    and the PC Terminal programm.                                    */
/*                                                                     */
/***********************************************************************/

/* == EXTERNALS ====================================================== */
/*                                                                     */
#include <inttypes.h>
#include <compat/twi.h>
#include "../misc/typedef.h"
#include "uart.h"
#include <util/delay.h>
#include <avr/interrupt.h>
#include "../lcd/lcd.h"
#include <avr/io.h>
#include <string.h>
#include <stdlib.h>


/* == FUNCTION PROTOTYPES ============================================ */
/*                                                                     */
void uart1_init(unsigned int baudrate);
unsigned int uart1_getc(void);
void uart1_putc(unsigned char data);
void uart1_puts(const char *s );
U_INT8 uart1_getc_ne(void);
U_INT8 uart1_conv_getc(U_INT16 uart1_received_int);

void   rtc_settime(U_INT8 seconds, U_INT8 minutes, U_INT8 hours);
U_INT8 rtc_readsec(void);
U_INT8 rtc_readmin(void);
U_INT8 rtc_readhour(void);


/* == FUNCTION uart1_send_data ======================================= */
/*                                                                     */
/*    This function sends one data package to the terminal programm    */
/*    One package possesses the following mask:                        */
/*    DDmmsstttcccfffp                                                 */
/*    DD  - A fixed string (shows that the package is a data package)  */
/*    mm  - Past minutes since the process has started                 */
/*    ss  - Past seconds since the process has started                 */
/*    ttt - Temperature (at the time the function is called)           */
/*    ccc - Temperature (reference from the equation)                  */
/*    fff - Progress (in percent)                                      */
/*    p   - Phase (at the time the function is called)                 */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    U_INT16 temp_soll : Reference Temperature                        */
/*    U_INT16 temp_ist  : Temperature at the moment                    */
/*    U_INT8  progress  : Progress (in percent)                        */
/*    U_INT8  phase     : Phase (at the time the function is called)   */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    none															   */
/*                                                                     */
/***********************************************************************/
void uart1_send_data(U_INT16 temp_soll, U_INT16 temp_ist , U_INT8 progress, U_INT8 phase)
{
	//Define the different parts of a package
	char package_source[25];
	char package_begin[3] = "#DD";
	char package_minutes[3];
	char package_seconds[3];
	char package_tempist[3];
	char package_tempsoll[3];
	char package_progress[3];
	char package_phase[1];
	char package_end[1]   = "-";

	U_INT8 minutes, seconds;

	//Read minutes and seconds directly from the RTC
	seconds = rtc_readsec();
	minutes = rtc_readmin();

	//Convert the integer values to ASCII code
	itoa(temp_soll,package_tempsoll,10);
	itoa(temp_ist ,package_tempist ,10);
	itoa(progress ,package_progress,10);
	itoa(phase    ,package_phase   ,10);
	itoa(minutes  ,package_minutes ,10);
	itoa(seconds  ,package_seconds ,10);

	//Copy the parts of the package to the end of the source package
	strncat(package_source,package_begin   ,3);
	strncat(package_source,package_minutes ,3);
	strncat(package_source,package_seconds ,3);
	strncat(package_source,package_tempist ,3);
	strncat(package_source,package_tempsoll,3);
	strncat(package_source,package_progress,3);
	strncat(package_source,package_phase   ,1);
	strncat(package_source,package_end     ,1);

	//Send the complete package 
	uart1_puts(package_source);
}

/* == FUNCTION uart1_getc_ne ========================================= */
/*                                                                     */
/*    This function gets a char out of the UART1 buffer WITHOUT the    */
/*    last recognized error.										   */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    none                                                             */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    U_INT8 : last received char from UART1 buffer   				   */
/*                                                                     */
/***********************************************************************/
U_INT8 uart1_getc_ne(void)
{
	U_INT16 uart1_received_int;
	U_INT8 uart1_received_char;
	uart1_received_int = uart1_getc();
	uart1_received_char = (U_INT8) (uart1_received_int & 0x00FF);

	return uart1_received_char;
}

/* == FUNCTION uart1_conv_getc ======================================= */
/*                                                                     */
/*    This function converts the return value of the uart1_getc   	   */
/*    into a single byte that contains the last received character.    */
/*                                                                     */
/* -- PARAMETERS ----------------------------------------------------- */
/*                                                                     */
/*    U_INT6 : return value from uart1_getc()                          */
/*                                                                     */
/* -- RETURN VALUE --------------------------------------------------- */
/*                                                                     */
/*    U_INT8 : last received character from the return value		   */
/*                                                                     */
/***********************************************************************/
U_INT8 uart1_conv_getc(U_INT16 uart1_received_int)
{
	U_INT8 converted_char;
	converted_char = (U_INT8) (uart1_received_int & 0x00FF);

	return converted_char;
}




/** END OF MODULE ******************************************************/
